@@ -34,11 +34,7 @@
 
 namespace Twitterizer
 {
-    using System;
-    using System.Collections.Generic;
-    using System.Linq;
-    using System.Text;
-    using Twitterizer.Core;
+    using Core;
 
     /// <summary>
     /// Provides interaction with the Twitter API to obtain and manage relationships between users.
@@ -54,15 +50,11 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatusCollection"/> instance.
         /// </returns>
-        public static TwitterUserCollection Followers(OAuthTokens tokens, FollowersOptions options)
+        public static TwitterResponse<TwitterUserCollection> Followers(OAuthTokens tokens, FollowersOptions options)
         {
             Commands.FollowersCommand command = new Commands.FollowersCommand(tokens, options);
 
-            TwitterUserCollection result = Core.CommandPerformer<TwitterUserCollection>.PerformAction(command);
-
-            result.CursorPagedCommand = command;
-
-            return result;
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -72,7 +64,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatusCollection"/> instance.
         /// </returns>
-        public static TwitterUserCollection Followers(OAuthTokens tokens)
+        public static TwitterResponse<TwitterUserCollection> Followers(OAuthTokens tokens)
         {
             return Followers(tokens, null);
         }
@@ -84,7 +76,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterStatusCollection"/> instance.
         /// </returns>
-        public static TwitterUserCollection Followers(FollowersOptions options)
+        public static TwitterResponse<TwitterUserCollection> Followers(FollowersOptions options)
         {
             return Followers(null, options);
         }
@@ -101,14 +93,12 @@ namespace Twitterizer
         /// A <see cref="TwitterUserCollection"/> instance.
         /// </returns>
         /// <remarks>Please note that the result set isn't guaranteed to be 100 every time as suspended users will be filtered out.</remarks>
-        public static TwitterUserCollection Friends(OAuthTokens tokens, FriendsOptions options)
+        [System.Obsolete("This method is deprecated as it will only return information about users who have Tweeted recently. It is not a functional way to retrieve all of a users friends. Instead of using this method use a combination of friends/ids and users/lookup.")]
+        public static TwitterResponse<TwitterUserCollection> Friends(OAuthTokens tokens, FriendsOptions options)
         {
             Commands.FriendsCommand command = new Commands.FriendsCommand(tokens, options);
 
-            TwitterUserCollection result = Core.CommandPerformer<TwitterUserCollection>.PerformAction(command);
-            result.CursorPagedCommand = command;
-
-            return result;
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -119,7 +109,8 @@ namespace Twitterizer
         /// A <see cref="TwitterUserCollection"/> instance.
         /// </returns>
         /// <remarks>Please note that the result set isn't guaranteed to be 100 every time as suspended users will be filtered out.</remarks>
-        public static TwitterUserCollection Friends(OAuthTokens tokens)
+        [System.Obsolete("This method is deprecated as it will only return information about users who have Tweeted recently. It is not a functional way to retrieve all of a users friends. Instead of using this method use a combination of friends/ids and users/lookup.")]
+        public static TwitterResponse<TwitterUserCollection> Friends(OAuthTokens tokens)
         {
             return Friends(tokens, null);
         }
@@ -132,7 +123,8 @@ namespace Twitterizer
         /// A <see cref="TwitterUserCollection"/> instance.
         /// </returns>
         /// <remarks>Please note that the result set isn't guaranteed to be 100 every time as suspended users will be filtered out.</remarks>
-        public static TwitterUserCollection Friends(FriendsOptions options)
+        [System.Obsolete("This method is deprecated as it will only return information about users who have Tweeted recently. It is not a functional way to retrieve all of a users friends. Instead of using this method use a combination of friends/ids and users/lookup.")]
+        public static TwitterResponse<TwitterUserCollection> Friends(FriendsOptions options)
         {
             return Friends(null, options);
         }
@@ -148,7 +140,7 @@ namespace Twitterizer
         /// <returns>
         /// Returns the followed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Create(OAuthTokens tokens, decimal userId)
+        public static TwitterResponse<TwitterUser> Create(OAuthTokens tokens, decimal userId)
         {
             return Create(tokens, userId, null);
         }
@@ -162,10 +154,10 @@ namespace Twitterizer
         /// <returns>
         /// Returns the followed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Create(OAuthTokens tokens, decimal userId, CreateFriendshipOptions options)
+        public static TwitterResponse<TwitterUser> Create(OAuthTokens tokens, decimal userId, CreateFriendshipOptions options)
         {
             Commands.CreateFriendshipCommand command = new Commands.CreateFriendshipCommand(tokens, userId, options);
-            return CommandPerformer<TwitterUser>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -176,7 +168,7 @@ namespace Twitterizer
         /// <returns>
         /// Returns the followed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Create(OAuthTokens tokens, string userName)
+        public static TwitterResponse<TwitterUser> Create(OAuthTokens tokens, string userName)
         {
             return Create(tokens, userName, null);
         }
@@ -190,10 +182,10 @@ namespace Twitterizer
         /// <returns>
         /// Returns the followed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Create(OAuthTokens tokens, string userName, CreateFriendshipOptions options)
+        public static TwitterResponse<TwitterUser> Create(OAuthTokens tokens, string userName, CreateFriendshipOptions options)
         {
             Commands.CreateFriendshipCommand command = new Commands.CreateFriendshipCommand(tokens, userName, options);
-            return CommandPerformer<TwitterUser>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         #endregion
@@ -208,7 +200,7 @@ namespace Twitterizer
         /// <returns>
         /// Returns the unfollowed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Delete(OAuthTokens tokens, decimal userId)
+        public static TwitterResponse<TwitterUser> Delete(OAuthTokens tokens, decimal userId)
         {
            return Delete(tokens, userId, null);
         }
@@ -222,10 +214,10 @@ namespace Twitterizer
         /// <returns>
         /// Returns the unfollowed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Delete(OAuthTokens tokens, decimal userId, OptionalProperties options)
+        public static TwitterResponse<TwitterUser> Delete(OAuthTokens tokens, decimal userId, OptionalProperties options)
         {
             Commands.DeleteFriendshipCommand command = new Commands.DeleteFriendshipCommand(tokens, userId, string.Empty, options);
-            return Core.CommandPerformer<TwitterUser>.PerformAction(command);
+            return CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -236,7 +228,7 @@ namespace Twitterizer
         /// <returns>
         /// Returns the unfollowed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Delete(OAuthTokens tokens, string userName)
+        public static TwitterResponse<TwitterUser> Delete(OAuthTokens tokens, string userName)
         {
             return Delete(tokens, userName, null);
         }
@@ -250,10 +242,10 @@ namespace Twitterizer
         /// <returns>
         /// Returns the unfollowed user in the requested format when successful.
         /// </returns>
-        public static TwitterUser Delete(OAuthTokens tokens, string userName, OptionalProperties options)
+        public static TwitterResponse<TwitterUser> Delete(OAuthTokens tokens, string userName, OptionalProperties options)
         {
             Commands.DeleteFriendshipCommand command = new Commands.DeleteFriendshipCommand(tokens, 0, userName, options);
-            return Core.CommandPerformer<TwitterUser>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
         #endregion
 
@@ -265,7 +257,7 @@ namespace Twitterizer
         /// <param name="tokens">The tokens.</param>
         /// <param name="targetUserId">The target user id.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, decimal targetUserId)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, decimal targetUserId)
         {
             return Show(tokens, targetUserId, null);
         }
@@ -277,7 +269,7 @@ namespace Twitterizer
         /// <param name="targetUserId">The target user id.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, decimal targetUserId, OptionalProperties options)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, decimal targetUserId, OptionalProperties options)
         {
             return Show(tokens, 0, targetUserId, options);
         }
@@ -289,7 +281,7 @@ namespace Twitterizer
         /// <param name="sourceUseId">The source user id.</param>
         /// <param name="targetUserId">The target user id.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, decimal sourceUseId, decimal targetUserId)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, decimal sourceUseId, decimal targetUserId)
         {
             return Show(tokens, sourceUseId, targetUserId, null);
         }
@@ -302,7 +294,7 @@ namespace Twitterizer
         /// <param name="targetUserId">The target user id.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, decimal sourceUseId, decimal targetUserId, OptionalProperties options)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, decimal sourceUseId, decimal targetUserId, OptionalProperties options)
         {
             Commands.ShowFriendshipCommand command = new Twitterizer.Commands.ShowFriendshipCommand(
                 tokens, 
@@ -312,7 +304,7 @@ namespace Twitterizer
                 string.Empty, 
                 options);
 
-            return Core.CommandPerformer<TwitterRelationship>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -321,7 +313,7 @@ namespace Twitterizer
         /// <param name="tokens">The tokens.</param>
         /// <param name="targetUserName">The target user name.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, string targetUserName)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, string targetUserName)
         {
             return Show(tokens, string.Empty, targetUserName, null);
         }
@@ -333,7 +325,7 @@ namespace Twitterizer
         /// <param name="targetUserName">The target user name.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, string targetUserName, OptionalProperties options)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, string targetUserName, OptionalProperties options)
         {
             return Show(tokens, string.Empty, targetUserName, options);
         }
@@ -345,7 +337,7 @@ namespace Twitterizer
         /// <param name="sourceUserName">The source user name.</param>
         /// <param name="targetUserName">The target user name.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, string sourceUserName, string targetUserName)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, string sourceUserName, string targetUserName)
         {
             return Show(tokens, sourceUserName, targetUserName, null);
         }
@@ -358,11 +350,11 @@ namespace Twitterizer
         /// <param name="targetUserName">The target user name.</param>
         /// <param name="options">The options.</param>
         /// <returns>A <see cref="TwitterRelationship"/> instance.</returns>
-        public static TwitterRelationship Show(OAuthTokens tokens, string sourceUserName, string targetUserName, OptionalProperties options)
+        public static TwitterResponse<TwitterRelationship> Show(OAuthTokens tokens, string sourceUserName, string targetUserName, OptionalProperties options)
         {
             Commands.ShowFriendshipCommand command = new Twitterizer.Commands.ShowFriendshipCommand(tokens, 0, sourceUserName, 0, targetUserName, options);
 
-            return Core.CommandPerformer<TwitterRelationship>.PerformAction(command);
+            return Core.CommandPerformer.PerformAction(command);
         }
 
         /// <summary>
@@ -373,7 +365,7 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterRelationship"/> instance.
         /// </returns>
-        public static TwitterRelationship Show(decimal sourceUseId, decimal targetUserId)
+        public static TwitterResponse<TwitterRelationship> Show(decimal sourceUseId, decimal targetUserId)
         {
             return Show(null, sourceUseId, targetUserId, null);
         }
@@ -386,11 +378,112 @@ namespace Twitterizer
         /// <returns>
         /// A <see cref="TwitterRelationship"/> instance.
         /// </returns>
-        public static TwitterRelationship Show(string sourceUserName, string targetUserName)
+        public static TwitterResponse<TwitterRelationship> Show(string sourceUserName, string targetUserName)
         {
             return Show(null, sourceUserName, targetUserName, null);
         }
 
         #endregion
+
+        #region User IDs lists
+        
+        /// <summary>
+        /// Returns the numeric IDs for every user the specified user is friends with.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>
+        /// A <see cref="TwitterListCollection"/> instance.
+        /// </returns>
+        public static TwitterResponse<UserIdCollection> FriendsIds(OAuthTokens tokens, UsersIdsOptions options)
+        {
+            Commands.FriendsIdsCommand command = new Commands.FriendsIdsCommand(tokens, options);
+            return Core.CommandPerformer.PerformAction(command);
+        }
+
+        /// <summary>
+        /// Returns the numeric IDs for every user the specified user is friends with.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <returns>
+        /// A <see cref="TwitterListCollection"/> instance.
+        /// </returns>
+        public static TwitterResponse<UserIdCollection> FriendsIds(OAuthTokens tokens)
+        {
+            return FriendsIds(tokens, null);
+        }
+
+        /// <summary>
+        /// Returns the numeric IDs for every user the specified user is following.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="options">The options.</param>
+        /// <returns>
+        /// A <see cref="TwitterListCollection"/> instance.
+        /// </returns>
+        public static TwitterResponse<UserIdCollection> FollowersIds(OAuthTokens tokens, UsersIdsOptions options)
+        {
+            Commands.FollowersIdsCommand command = new Commands.FollowersIdsCommand(tokens, options);
+            return Core.CommandPerformer.PerformAction(command);
+        }
+
+        /// <summary>
+        /// Returns the numeric IDs for every user the specified user is following.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <returns>
+        /// A <see cref="TwitterListCollection"/> instance.
+        /// </returns>
+        public static TwitterResponse<UserIdCollection> FollowersIds(OAuthTokens tokens)
+        {
+            return FollowersIds(tokens, null);
+        }
+
+        #endregion
+
+        /// <summary>
+        /// Returns a collection of IDs for every user who has a pending request to follow the authenticating user.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="options">The options.</param>
+        /// <returns></returns>
+        public static TwitterResponse<TwitterCursorPagedIdCollection> IncomingRequests(OAuthTokens tokens, IncomingFriendshipsOptions options)
+        {
+            Commands.IncomingFriendshipsCommand command = new Commands.IncomingFriendshipsCommand(tokens, options);
+            return Core.CommandPerformer.PerformAction(command);
+        }
+
+        /// <summary>
+        /// Returns a collection of IDs for every user who has a pending request to follow the authenticating user.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <returns></returns>
+        public static TwitterResponse<TwitterCursorPagedIdCollection> IncomingRequests(OAuthTokens tokens)
+        {
+            return IncomingRequests(tokens, null);
+        }
+
+        /// <summary>
+        /// Returns a collection of IDs for every protected user for whom the authenticating user has a pending follow request.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <param name="options">The options.</param>
+        /// <returns></returns>
+        public static TwitterResponse<TwitterCursorPagedIdCollection> OutgoingRequests(OAuthTokens tokens, OutgoingFriendshipsOptions options)
+        {
+            Commands.OutgoingFriendshipsCommand command = new Commands.OutgoingFriendshipsCommand(tokens, options);
+            return Core.CommandPerformer.PerformAction(command);
+        }
+
+        /// <summary>
+        /// Returns a collection of IDs for every protected user for whom the authenticating user has a pending follow request.
+        /// </summary>
+        /// <param name="tokens">The tokens.</param>
+        /// <returns></returns>
+        public static TwitterResponse<TwitterCursorPagedIdCollection> OutgoingRequests(OAuthTokens tokens)
+        {
+            return OutgoingRequests(tokens, null);
+        }
+
     }
 }
