@@ -3,6 +3,9 @@
     using System.Collections.Generic;
     using System.Collections.ObjectModel;
 
+    /// <summary>
+    /// Provides optional parameters for user lookup methods.
+    /// </summary>
     public class LookupUsersOptions : OptionalProperties
     {
         /// <summary>
@@ -11,7 +14,7 @@
         public LookupUsersOptions()
         {
             this.ScreenNames = new Collection<string>();
-            this.UserIds = new Collection<decimal>();
+            this.UserIds = new TwitterIdCollection();
         }
 
         /// <summary>
@@ -24,7 +27,7 @@
         /// Gets or sets the user ids.
         /// </summary>
         /// <value>The user ids.</value>
-        public Collection<decimal> UserIds { get; set; }
+        public TwitterIdCollection UserIds { get; set; }
 
         /// <summary>
         /// Gets or sets a value indicating whether [include entities].
