@@ -34,6 +34,7 @@
 namespace Twitterizer
 {
     using System;
+    using System.IO;
     using System.Collections.Generic;
     using System.Globalization;
     using System.Linq;
@@ -41,10 +42,14 @@ namespace Twitterizer
     using System.Security.Cryptography;
     using System.Text;
     using System.Text.RegularExpressions;
-    using System.Web;
+#if SILVERLIGHT
+    using System.Net.Browser;
+    using System.Threading;
+    using System.Windows.Threading;
+#endif
 
     /// <summary>
-    /// Enumeration of the supported HTTP verbs supported by the <see cref="Twitterizer.Core.CommandPerformer{T}"/>
+    /// Enumeration of the supported HTTP verbs supported by the <see cref="Twitterizer.Core.CommandPerformer"/>
     /// </summary>
     public enum HTTPVerb
     {
@@ -69,6 +74,11 @@ namespace Twitterizer
     /// </summary>
     public sealed class WebRequestBuilder
     {
+        /// <summary>
+        /// Holds file data form performing multipart form posts.
+        /// </summary>
+        private byte[] formData = null;
+
         /// <summary>
         /// The HTTP Authorization realm.
         /// </summary>
@@ -84,7 +94,7 @@ namespace Twitterizer
         /// Gets or sets the parameters.
         /// </summary>
         /// <value>The parameters.</value>
-        public Dictionary<string, string> Parameters { get; private set; }
+		public Dictionary<string, object> Parameters { get; private set; }
 
         /// <summary>
         /// Gets or sets the verb.
@@ -96,13 +106,33 @@ namespace Twitterizer
         /// Gets or sets the oauth tokens.
         /// </summary>
         /// <value>The tokens.</value>
-        public OAuthTokens Tokens { get; set; }
+        public OAuthTokens Tokens { private get; set; }
 
+        /// <summary>
+        /// Gets or sets the UserAgent.
+        /// </summary>
+        /// <value>The User Agent.</value>
+        public String UserAgent { private get; set; }
+
+        /// <summary>
+        /// Gets or sets the Basic Auth Credentials.
+        /// </summary>
+        /// <value>The Basic Auth Credentials.</value>
+		public NetworkCredential NetworkCredentials { private get; set; }
+
+		/// <summary>
+		/// Gets or sets the Multipart config
+		/// </summary>
+		/// <value>Multipart</value>
+		public bool Multipart { get; set; }
+
+#if !SILVERLIGHT
         /// <summary>
         /// Gets or sets the proxy.
         /// </summary>
         /// <value>The proxy.</value>
         public WebProxy Proxy { get; set; }
+#endif
 
         /// <summary>
         /// Gets or sets a value indicating whether the request will be signed with an OAuth authorization header.
@@ -120,7 +150,8 @@ namespace Twitterizer
                                                               "oauth_timestamp",
                                                               "oauth_signature_method",
                                                               "oauth_consumer_key",
-                                                              "oauth_token"
+                                                              "oauth_token",
+                                                              "oauth_verifier"
                                                               // Leave signature omitted from the list, it is added manually
                                                               // "oauth_signature",
                                                           };
@@ -139,17 +170,24 @@ namespace Twitterizer
         /// Initializes a new instance of the <see cref="WebRequestBuilder"/> class.
         /// </summary>
         /// <param name="requestUri">The request URI.</param>
-        /// <param name="verb">The verb.</param>
-        public WebRequestBuilder(Uri requestUri, HTTPVerb verb)
+        /// <param name="verb">The http verb.</param>
+        /// <param name="KeepAlive">if set to <c>true</c> the <see cref="System.Net.HttpWebRequest"/> will be instructed to keep the connection alive.</param>
+        /// <param name="UserAgent">The http user agent.</param>
+        /// <param name="NetworkCredentials">The network credentials.</param>
+        /// <remarks></remarks>
+        public WebRequestBuilder(Uri requestUri, HTTPVerb verb, String UserAgent, NetworkCredential NetworkCredentials)
         {
             if (requestUri == null)
                 throw new ArgumentNullException("requestUri");
 
             this.RequestUri = requestUri;
             this.Verb = verb;
+            this.UserAgent = UserAgent;
             this.UseOAuth = false;
+            if (NetworkCredentials != null)
+                this.NetworkCredentials = NetworkCredentials;
 
-            this.Parameters = new Dictionary<string, string>();
+			this.Parameters = new Dictionary<string, object>();
 
             if (string.IsNullOrEmpty(this.RequestUri.Query)) return;
 
@@ -167,8 +205,10 @@ namespace Twitterizer
         /// <param name="requestUri">The request URI.</param>
         /// <param name="verb">The verb.</param>
         /// <param name="tokens">The tokens.</param>
-        public WebRequestBuilder(Uri requestUri, HTTPVerb verb, OAuthTokens tokens)
-            : this(requestUri, verb)
+        /// <param name="KeepAlive">if set to <c>true</c> the http request is instructed to keep the connection alive.</param>
+        /// <param name="UserAgent">The user agent.</param>
+        public WebRequestBuilder(Uri requestUri, HTTPVerb verb, OAuthTokens tokens, String UserAgent = "")
+            : this(requestUri, verb, UserAgent, null)
         {
             this.Tokens = tokens;
 
@@ -196,7 +236,31 @@ namespace Twitterizer
         {
             HttpWebRequest request = PrepareRequest();
 
+#if !SILVERLIGHT
             return (HttpWebResponse)request.GetResponse();
+#else
+            request.AllowReadStreamBuffering = true;
+            HttpWebResponse response = null;
+            AutoResetEvent alldone = new AutoResetEvent(false);
+            IAsyncResult asyncResult = request.BeginGetResponse(new AsyncCallback((param) => 
+            {
+                HttpWebRequest req = (HttpWebRequest)param.AsyncState;
+                try
+                {
+                    response = (HttpWebResponse)req.EndGetResponse(param);
+                }
+                catch (WebException we)
+                {
+                    response = (HttpWebResponse)we.Response;
+                }
+                finally
+                {
+                    alldone.Set();
+                }
+            }), request);
+            alldone.WaitOne();
+            return response;
+#endif
         }
 
         /// <summary>
@@ -206,22 +270,84 @@ namespace Twitterizer
         public HttpWebRequest PrepareRequest()
         {
             SetupOAuth();
-            AddQueryStringParametersToUri();
 
+			formData = null;
+			string contentType = string.Empty;
+
+			if (!Multipart)
+			{	//We don't add the paramters to the query if we are multipart-ing
+				AddQueryStringParametersToUri();
+			}
+			else
+			{
+				string dataBoundary = "--------------------r4nd0m";
+				contentType = "multipart/form-data; boundary=" + dataBoundary;
+
+				formData = GetMultipartFormData(Parameters, dataBoundary);
+
+				this.Verb = HTTPVerb.POST;
+			}
+#if SILVERLIGHT
+            WebRequest.RegisterPrefix("http://", WebRequestCreator.ClientHttp);
+            WebRequest.RegisterPrefix("https://", WebRequestCreator.ClientHttp);
+#endif
             HttpWebRequest request = (HttpWebRequest)WebRequest.Create(this.RequestUri);
+
+#if !SILVERLIGHT
             if (this.Proxy != null)
                 request.Proxy = Proxy;
-            request.Method = this.Verb.ToString();
-            request.UserAgent = string.Format(CultureInfo.InvariantCulture, "Twitterizer/{0}", System.Reflection.Assembly.GetAssembly(typeof(WebRequestBuilder)).GetName().Version);
-            request.ServicePoint.Expect100Continue = false;
-            
-            if (this.UseOAuth)
+#endif
+            if (!this.UseOAuth && this.NetworkCredentials != null)
             {
-                request.Headers.Add("Authorization", GenerateAuthorizationHeader());
+                request.Credentials = this.NetworkCredentials;
+                request.UseDefaultCredentials = false;
+            }
+            else
+            {
+                request.UseDefaultCredentials = true;
             }
 
-            //AddFormFieldValuesToRequest(request);
+            request.Method = this.Verb.ToString();
+
+			request.ContentLength = Multipart ? formData.Length : 0;
+
+
+#if !SILVERLIGHT // No silverlight user-agent as Assembly.GetName() isn't supported and setting the request.UserAgent is also not supported.
+            request.UserAgent = (String.IsNullOrEmpty(UserAgent)) ? string.Format(CultureInfo.InvariantCulture, "Twitterizer/{0}", System.Reflection.Assembly.GetExecutingAssembly().GetName().Version) : UserAgent;
             
+            request.ServicePoint.Expect100Continue = false;
+#endif
+            if (this.UseOAuth)
+            {
+#if !SILVERLIGHT
+                request.Headers.Add("Authorization", GenerateAuthorizationHeader());
+#else
+                request.Headers["Authorization"] = GenerateAuthorizationHeader();
+#endif
+            }
+
+			if (Multipart)
+			{	//Parameters are not added to the query string, post them in the request body instead
+
+				request.ContentType = contentType;
+
+
+#if !SILVERLIGHT
+                using (Stream requestStream = request.GetRequestStream())
+#else
+                IAsyncResult getRequestStreamResult = request.BeginGetRequestStream(
+                    (res) =>
+                    {
+
+                    }, null);
+
+                using (Stream requestStream = request.EndGetRequestStream(getRequestStreamResult))
+#endif
+                {
+                    requestStream.Write(formData, 0, formData.Length);
+                }
+			}
+
             return request;
         }
 
@@ -233,14 +359,14 @@ namespace Twitterizer
             StringBuilder requestParametersBuilder = new StringBuilder(this.RequestUri.AbsoluteUri);
             requestParametersBuilder.Append(this.RequestUri.Query.Length == 0 ? "?" : "&");
 
-            var fieldsToInclude = from p in this.Parameters
-                                  where !OAuthParametersToIncludeInHeader.Contains(p.Key) &&
-                                        !SecretParameters.Contains(p.Key)
-                                  select p;
 
-            foreach (KeyValuePair<string, string> item in fieldsToInclude)
+			Dictionary<string, object> fieldsToInclude = new Dictionary<string, object>(this.Parameters.Where(p => !OAuthParametersToIncludeInHeader.Contains(p.Key) &&
+                                         !SecretParameters.Contains(p.Key)).ToDictionary(p => p.Key, p => p.Value));
+
+			foreach (KeyValuePair<string, object> item in fieldsToInclude)
             {
-                requestParametersBuilder.AppendFormat("{0}={1}&", item.Key, UrlEncode(item.Value));
+				if( item.Value.GetType() == typeof(string) )
+					requestParametersBuilder.AppendFormat("{0}={1}&", item.Key, UrlEncode(item.Value as string));
             }
 
             if (requestParametersBuilder.Length == 0)
@@ -251,45 +377,48 @@ namespace Twitterizer
             this.RequestUri = new Uri(requestParametersBuilder.ToString());
         }
 
-        /// <summary>
-        /// Adds the form field values to request.
-        /// </summary>
-        /// <param name="request">The request.</param>
-        private void AddFormFieldValuesToRequest(WebRequest request)
-        {
-            throw new NotImplementedException("Multipart form data is not yet supported by the WebRequestBuilder.");
+		private byte[] GetMultipartFormData(Dictionary<string, object> param, string boundary)
+		{
+			Stream formDataStream = new MemoryStream();
+			Encoding encoding = Encoding.UTF8;
 
-/*
-            if (!(new[] { HTTPVerb.DELETE, HTTPVerb.POST }).Contains(this.Verb))
-                return;
+			Dictionary<string, object> fieldsToInclude = new Dictionary<string, object>(param.Where(p => !OAuthParametersToIncludeInHeader.Contains(p.Key) &&
+							 !SecretParameters.Contains(p.Key)).ToDictionary(p => p.Key, p => p.Value));
 
-            request.ContentType = "application/x-www-form-urlencoded";
+			foreach (KeyValuePair<string, object> kvp in fieldsToInclude)
+			{
+				if (kvp.Value.GetType() == typeof(byte[]))
+				{	//assume this to be a byte stream
+					byte[] data = kvp.Value as byte[];
 
-            StringBuilder requestParametersBuilder = new StringBuilder();
+					string header = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"; filename=\"{2}\";\r\nContent-Type: application/octet-stream\r\n\r\n",
+						boundary,
+						kvp.Key,
+						kvp.Key);
+					formDataStream.Write(encoding.GetBytes(header), 0, header.Length);
+					formDataStream.Write(data, 0, data.Length);
+				}
+				else
+				{	//this is normal text data
+					string header = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"\r\n\r\n{2}\r\n",
+						boundary,
+						kvp.Key,
+						kvp.Value);
+					formDataStream.Write(encoding.GetBytes(header), 0, header.Length);
+				}
+			}
 
-            var fieldsToInclude = from p in this.Parameters
-                                  where !OAuthParametersToIncludeInHeader.Contains(p.Key) &&
-                                        !SecretParameters.Contains(p.Key)
-                                  select p;
+			string footer = string.Format("\r\n--{0}--\r\n", boundary);
+			formDataStream.Write(encoding.GetBytes(footer), 0, footer.Length);
+			formDataStream.Position = 0;
+			byte[] returndata = new byte[formDataStream.Length];
 
-            foreach (KeyValuePair<string, string> item in fieldsToInclude)
-            {
-                requestParametersBuilder.AppendFormat("{0}={1}&", item.Key, item.Value);
-            }
+			formDataStream.Read(returndata, 0, returndata.Length);
+			formDataStream.Close();
 
-            if (requestParametersBuilder.Length == 0)
-                return;
+			return returndata;
+		}
 
-            requestParametersBuilder.Remove(requestParametersBuilder.Length - 1, 1);
-
-            byte[] formData = Encoding.UTF8.GetBytes(requestParametersBuilder.ToString());
-            request.ContentLength = formData.Length;
-            System.IO.Stream requestStream = request.GetRequestStream();
-            requestStream.Write(formData, 0, formData.Length);
-            requestStream.Close();
-*/
-        }
-        
         #region OAuth Helper Methods
         /// <summary>
         /// Sets up the OAuth request details.
@@ -332,16 +461,20 @@ namespace Twitterizer
         /// <returns></returns>
         public string GenerateSignature()
         {
-            var nonSecretParameters = (from p in this.Parameters
-                                       where !SecretParameters.Contains(p.Key)
-                                       select p);
+			IEnumerable<KeyValuePair<string, object>> nonSecretParameters = null;
 
-            var baseStringParameters =
-                (from p in this.Parameters
-                 where !(p.Key.EndsWith("_secret", StringComparison.OrdinalIgnoreCase) &&
-                         p.Key.StartsWith("oauth_", StringComparison.OrdinalIgnoreCase) &&
-                         !p.Key.EndsWith("_verifier", StringComparison.OrdinalIgnoreCase))
-                 select p);
+			if (Multipart)
+			{
+				nonSecretParameters = (from p in this.Parameters
+										   where (!SecretParameters.Contains(p.Key) && p.Key.StartsWith("oauth_"))
+										   select p);
+			}
+			else
+			{
+				nonSecretParameters = (from p in this.Parameters
+										   where (!SecretParameters.Contains(p.Key))
+										   select p);
+			}
 
             Uri urlForSigning = this.RequestUri;
 
@@ -351,7 +484,7 @@ namespace Twitterizer
                 "{0}&{1}&{2}",
                 this.Verb.ToString().ToUpper(CultureInfo.InvariantCulture),
                 UrlEncode(NormalizeUrl(urlForSigning)),
-                UrlEncode(baseStringParameters));
+                UrlEncode(nonSecretParameters));
 
             // Create our hash key (you might say this is a password)
             string key = string.Format(
@@ -362,8 +495,8 @@ namespace Twitterizer
 
 
             // Generate the hash
-            HMACSHA1 hmacsha1 = new HMACSHA1(Encoding.ASCII.GetBytes(key));
-            byte[] signatureBytes = hmacsha1.ComputeHash(Encoding.ASCII.GetBytes(signatureBaseString));
+            HMACSHA1 hmacsha1 = new HMACSHA1(Encoding.UTF8.GetBytes(key));
+            byte[] signatureBytes = hmacsha1.ComputeHash(Encoding.UTF8.GetBytes(signatureBaseString));
             return Convert.ToBase64String(signatureBytes);
         }
 
@@ -419,7 +552,7 @@ namespace Twitterizer
                 return string.Empty;
             }
 
-            value = HttpUtility.UrlEncode(value).Replace("+", "%20");
+            value = Uri.EscapeDataString(value);
 
             // UrlEncode escapes with lowercase characters (e.g. %2f) but oAuth needs %2F
             value = Regex.Replace(value, "(%[0-9a-f][0-9a-f])", c => c.Value.ToUpper());
@@ -444,7 +577,7 @@ namespace Twitterizer
         /// </summary>
         /// <param name="parameters">The parameters.</param>
         /// <returns>A string of all the <paramref name="parameters"/> keys and value pairs with the values encoded.</returns>
-        private static string UrlEncode(IEnumerable<KeyValuePair<string, string>> parameters)
+		private static string UrlEncode(IEnumerable<KeyValuePair<string, object>> parameters)
         {
             StringBuilder parameterString = new StringBuilder();
 
@@ -459,12 +592,13 @@ namespace Twitterizer
                     parameterString.Append("&");
                 }
 
-                parameterString.Append(
-                    string.Format(
-                        CultureInfo.InvariantCulture,
-                        "{0}={1}",
-                        UrlEncode(item.Key),
-                        UrlEncode(item.Value)));
+				if( item.Value.GetType() == typeof(string) )
+					parameterString.Append(
+						string.Format(
+							CultureInfo.InvariantCulture,
+							"{0}={1}",
+							UrlEncode(item.Key),
+							UrlEncode(item.Value as string)));
             }
 
             return UrlEncode(parameterString.ToString());
@@ -477,11 +611,11 @@ namespace Twitterizer
         public string GenerateAuthorizationHeader()
         {
             StringBuilder authHeaderBuilder = new StringBuilder();
-            authHeaderBuilder.AppendFormat("OAuth realm=\"{0}\"", Realm);
+            authHeaderBuilder.AppendFormat("OAuth realm=\"\"", Realm);
 
             var sortedParameters = from p in this.Parameters
                                    where OAuthParametersToIncludeInHeader.Contains(p.Key)
-                                   orderby p.Key, UrlEncode(p.Value)
+                                   orderby p.Key, UrlEncode( (p.Value.GetType() == typeof(string) )? p.Value as string : "")
                                    select p;
 
             foreach (var item in sortedParameters)
@@ -489,15 +623,14 @@ namespace Twitterizer
                 authHeaderBuilder.AppendFormat(
                     ",{0}=\"{1}\"",
                     UrlEncode(item.Key),
-                    UrlEncode(item.Value));
+                    UrlEncode(item.Value as string));
             }
 
-            authHeaderBuilder.AppendFormat(",oauth_signature=\"{0}\"", UrlEncode(this.Parameters["oauth_signature"]));
+            authHeaderBuilder.AppendFormat(",oauth_signature=\"{0}\"", UrlEncode(this.Parameters["oauth_signature"] as string));
 
             return authHeaderBuilder.ToString();
         }
         #endregion
-
-
+        
     }
 }
