@@ -29,6 +29,7 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 using Mono.Unix.Native;
 #if CONFIG_NINI
 using Nini.Config;
@@ -283,8 +284,8 @@ namespace Smuxi.Engine
             Get(prefix+"BeepOnHighlight", false);
             
             prefix = "Engine/Users/DEFAULT/Connection/";
-            Get(prefix+"Encoding", String.Empty);
-            Get(prefix+"ProxyType", "None");
+            Get(prefix+"Encoding", "UTF-8");
+            Get(prefix+"ProxyType", "System");
             Get(prefix+"ProxyHostname", String.Empty);
             Get(prefix+"ProxyPort", -1);
             Get(prefix+"ProxyUsername", String.Empty);
@@ -294,6 +295,13 @@ namespace Smuxi.Engine
             Get(prefix+"Enabled", false);
             Get(prefix+"LogFilteredMessages", false);
 
+            prefix = "Engine/Users/DEFAULT/MessageBuffer/";
+            Get(prefix+"PersistencyType", "Volatile");
+            prefix = "Engine/Users/DEFAULT/MessageBuffer/Volatile/";
+            Get(prefix+"MaxCapacity", 200);
+            prefix = "Engine/Users/DEFAULT/MessageBuffer/Persistent/";
+            Get(prefix+"MaxCapacity", 50 * 1000);
+
             prefix = "Engine/Users/DEFAULT/Servers/";
             Get(prefix + "Servers", new string[] {
                 "IRC/irc.oftc.net",
@@ -404,6 +412,9 @@ namespace Smuxi.Engine
                 string realname = null;
                 try {
                     string gecos = Mono.Unix.UnixUserInfo.GetRealUser().RealName;
+                    if (gecos == null) {
+                        gecos = String.Empty;
+                    }
                     int pos = gecos.IndexOf(",");
                     if (pos != -1) {
                         realname = gecos.Substring(0, pos);
@@ -421,7 +432,7 @@ namespace Smuxi.Engine
                 LoadUserEntry(user, "Connection/Realname", realname);
                 LoadUserEntry(user, "Connection/Encoding", String.Empty);
 
-                LoadUserEntry(user, "Connection/ProxyType", "None");
+                LoadUserEntry(user, "Connection/ProxyType", "System");
                 LoadUserEntry(user, "Connection/ProxyHostname", String.Empty);
                 LoadUserEntry(user, "Connection/ProxyPort", -1);
                 LoadUserEntry(user, "Connection/ProxyUsername", null);
@@ -474,6 +485,10 @@ namespace Smuxi.Engine
                 LoadUserEntry(user, "Logging/Enabled", null);
                 LoadUserEntry(user, "Logging/LogFilteredMessages", null);
 
+                LoadUserEntry(user, "MessageBuffer/PersistencyType", null);
+                LoadUserEntry(user, "MessageBuffer/Volatile/MaxCapacity", null);
+                LoadUserEntry(user, "MessageBuffer/Persistent/MaxCapacity", null);
+
                 string[] servers = null;
                 string sprefix = prefix + user + "/Servers/";
                 servers = GetList(sprefix + "Servers");
@@ -518,7 +533,7 @@ namespace Smuxi.Engine
                     sprefix = prefix + user + "/Servers/" + server + "/";
                     LoadEntry(sprefix+"Hostname", null);
                     LoadEntry(sprefix+"Port", null);
-                    LoadEntry(sprefix+"Network", null);
+                    LoadEntry(sprefix+"Network", String.Empty);
                     LoadEntry(sprefix+"Encoding", null);
                     LoadEntry(sprefix+"Username", String.Empty);
                     LoadEntry(sprefix+"Password", String.Empty);
@@ -554,6 +569,17 @@ namespace Smuxi.Engine
             }
         }
 
+        public IDictionary<string, object> GetAll()
+        {
+            lock (m_Preferences) {
+                var dict = new Dictionary<string, object>(m_Preferences.Count);
+                foreach (DictionaryEntry entry in m_Preferences) {
+                    dict.Add((string) entry.Key, entry.Value);
+                }
+                return dict;
+            }
+        }
+
         public void Save()
         {
             Trace.Call();
