@@ -79,6 +79,9 @@ namespace Smuxi.Engine
 #endif
             
             // setting required default values
+            prefix = "Frontend/";
+            LoadEntry(prefix+"UseLowBandwidthMode", false);
+
             prefix = "Frontend/Engines/";
             Get<string[]>(prefix+"Engines", new string[] {});
             Get(prefix+"Default", String.Empty);
@@ -104,6 +107,7 @@ namespace Smuxi.Engine
                 LoadEntry(eprefix+"SshPort", 22);
                 LoadEntry(eprefix+"SshUsername", String.Empty);
                 LoadEntry(eprefix+"SshPassword", String.Empty);
+                LoadEntry(eprefix+"SshKeyfile", String.Empty);
             }
             
             LoadAllEntries("Frontend/"+_UIName);
