@@ -1,13 +1,7 @@
 /*
- * $Id: IrcProtocolManager.cs 149 2007-04-11 16:47:52Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Engine/IrcProtocolManager.cs $
- * $Rev: 149 $
- * $Author: meebey $
- * $Date: 2007-04-11 18:47:52 +0200 (Wed, 11 Apr 2007) $
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2007-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -112,11 +106,23 @@ namespace Smuxi.Engine
             }
         }
         
-        protected Session Session {
+        public virtual Session Session {
             get {
                 return _Session;
             }
         }
+
+        protected bool DebugProtocol {
+            get {
+#if LOG4NET
+                var repo = log4net.LogManager.GetRepository();
+                // info is higher than debug
+                return repo.Threshold <= log4net.Core.Level.Debug;
+#else
+                return false;
+#endif
+            }
+        }
         
         protected ProtocolManagerBase(Session session)
         {
@@ -140,8 +146,7 @@ namespace Smuxi.Engine
         
         public abstract bool Command(CommandModel cmd);
         public abstract void Connect(FrontendManager fm,
-                                     string hostname, int port,
-                                     string username, string password);
+                                     ServerModel server);
         public abstract void Reconnect(FrontendManager fm);
         public abstract void Disconnect(FrontendManager fm);
         
@@ -154,43 +159,29 @@ namespace Smuxi.Engine
 
         protected void NotConnected(CommandModel cmd)
         {
-            cmd.FrontendManager.AddTextToChat(
-                cmd.Chat,
-                String.Format(
-                    "-!- {0}",
-                    _("Not connected to server")
-                )
-            );
+            var msg = CreateMessageBuilder();
+            msg.AppendEventPrefix();
+            msg.AppendText(_("Not connected to server"));
+            cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg.ToMessage());
         }
 
         protected void NotEnoughParameters(CommandModel cmd)
         {
-            cmd.FrontendManager.AddTextToChat(
-                cmd.Chat,
-                String.Format(
-                    "-!- {0}",
-                    String.Format(
-                        _("Not enough parameters for {0} command"),
-                        cmd.Command
-                    )
-                )
-            );
+            var msg = CreateMessageBuilder();
+            msg.AppendEventPrefix();
+            msg.AppendText(_("Not enough parameters for {0} command"),
+                           cmd.Command);
+            cmd.FrontendManager.AddMessageToChat(cmd.Chat, msg.ToMessage());
         }
         
         protected virtual void OnConnected(EventArgs e)
         {
             Trace.Call(e);
 
-            Session.AddTextToChat(
-                Chat,
-                String.Format(
-                    "-!- {0}",
-                    String.Format(
-                        _("Connected to {0}"),
-                        NetworkID
-                    )
-                )
-            );
+            var msg = CreateMessageBuilder();
+            msg.AppendEventPrefix();
+            msg.AppendText(_("Connected to {0}"), NetworkID);
+            Session.AddMessageToChat(Chat, msg.ToMessage());
 
             _PresenceStatus = PresenceStatus.Online;
 
@@ -205,16 +196,10 @@ namespace Smuxi.Engine
         {
             Trace.Call(e);
 
-            Session.AddTextToChat(
-                Chat,
-                String.Format(
-                    "-!- {0}",
-                    String.Format(
-                        _("Disconnected from {0}"),
-                        NetworkID
-                    )
-                )
-            );
+            var msg = CreateMessageBuilder();
+            msg.AppendEventPrefix();
+            msg.AppendText(_("Disconnected from {0}"), NetworkID);
+            Session.AddMessageToChat(Chat, msg.ToMessage());
 
             _PresenceStatus = PresenceStatus.Offline;
 
@@ -267,5 +252,46 @@ namespace Smuxi.Engine
             
             return false;
         }
+
+        protected virtual void DebugRead(string data)
+        {
+            if (data == null) {
+                throw new ArgumentNullException("data");
+            }
+            if (Chat == null) {
+                return;
+            }
+            if (!DebugProtocol) {
+                return;
+            }
+
+            var msgBuilder = CreateMessageBuilder();
+            msgBuilder.MessageType = MessageType.Event;
+            // HACK: extra leading space to align with "-!- "
+            // HACK: extra trailing space to align with "WRITE: "
+            msgBuilder.AppendText("    READ:  ");
+            msgBuilder.AppendText(data);
+            Session.AddMessageToChat(Chat, msgBuilder.ToMessage());
+        }
+
+        protected virtual void DebugWrite(string data)
+        {
+            if (data == null) {
+                throw new ArgumentNullException("data");
+            }
+            if (Chat == null) {
+                return;
+            }
+            if (!DebugProtocol) {
+                return;
+            }
+
+            var msgBuilder = CreateMessageBuilder();
+            msgBuilder.MessageType = MessageType.Event;
+            // HACK: extra leading space to align with "-!- "
+            msgBuilder.AppendText("    WRITE: ");
+            msgBuilder.AppendText(data);
+            Session.AddMessageToChat(Chat, msgBuilder.ToMessage());
+        }
     }
 }
