@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2010 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2012 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -22,6 +22,8 @@
 
 using System;
 using System.IO;
+using System.Net;
+using System.Linq;
 using System.Text;
 using System.Text.RegularExpressions;
 using System.Collections;
@@ -41,7 +43,7 @@ namespace Smuxi.Engine
         private int                                   _Version = 0;
         private IDictionary<string, FrontendManager>  _FrontendManagers;
         private IList<IProtocolManager>               _ProtocolManagers;
-        private IList<ChatModel>                      _Chats;
+        private List<ChatModel>                       _Chats;
         private SessionChatModel                      _SessionChat;
         private Config                                _Config;
         private string                                _Username;
@@ -50,7 +52,12 @@ namespace Smuxi.Engine
         private FilterListController                  _FilterListController;
         private ICollection<FilterModel>              _Filters;
         private bool                                  _OnStartupCommandsProcessed;
-        
+        Timer NewsFeedTimer { get; set; }
+        List<string> SeenNewsFeedIds { get; set; }
+        DateTime NewsFeedLastModified { get; set; }
+        TimeSpan NewsFeedUpdateInterval { get; set; }
+        TimeSpan NewsFeedRetryInterval { get; set; }
+
         public IList<IProtocolManager> ProtocolManagers {
             get {
                 return _ProtocolManagers;
@@ -87,6 +94,12 @@ namespace Smuxi.Engine
             }
         }
         
+        public string Username {
+            get {
+                return _Username;
+            }
+        }
+
         public bool IsLocal {
             get {
                 return _Username == "local";
@@ -119,9 +132,21 @@ namespace Smuxi.Engine
             _FilterListController = new FilterListController(_UserConfig);
             _Filters = _FilterListController.GetFilterList().Values;
             _Chats = new List<ChatModel>();
-            
-            _SessionChat = new SessionChatModel("smuxi", "Smuxi");
-            _Chats.Add(_SessionChat);
+
+            InitSessionChat();
+
+            SeenNewsFeedIds = new List<string>();
+            NewsFeedUpdateInterval = TimeSpan.FromHours(12);
+            NewsFeedRetryInterval = TimeSpan.FromMinutes(5);
+            NewsFeedTimer = new Timer(delegate { UpdateNewsFeed(); }, null,
+                                      TimeSpan.Zero, NewsFeedUpdateInterval);
+        }
+
+        protected MessageBuilder CreateMessageBuilder()
+        {
+            var builder = new MessageBuilder();
+            builder.ApplyConfig(UserConfig);
+            return builder;
         }
 
         public void RegisterFrontendUI(IFrontendUI ui)
@@ -146,29 +171,6 @@ namespace Smuxi.Engine
             if (!_OnStartupCommandsProcessed) {
                 _OnStartupCommandsProcessed = true;
 
-                string str;
-                MessageModel msg;
-                msg = new MessageModel();
-                msg.MessageParts.Add(
-                    new TextMessagePartModel(new TextColor(0xFF0000), null, false,
-                            true, false, _("Welcome to Smuxi")));
-                AddMessageToChat(_SessionChat, msg);
-
-                msg = new MessageModel();
-                msg.MessageParts.Add(
-                    new TextMessagePartModel(null, null, false,
-                            true, false, _("Type /help to get a list of available commands.")));
-                AddMessageToChat(_SessionChat, msg);
-
-                str = _("After you have made a connection the list of " +
-                        "available commands changes. Use the /help command " +
-                        "again to see the extended command list.");
-                msg = new MessageModel();
-                msg.MessageParts.Add(
-                    new TextMessagePartModel(null, null, false,
-                            true, false, str));
-                AddMessageToChat(_SessionChat, msg);
-                
                 foreach (string command in (string[])_UserConfig["OnStartupCommands"]) {
                     if (command.Length == 0) {
                         continue;
@@ -359,6 +361,9 @@ namespace Smuxi.Engine
                 switch (cd.Command) {
                     case "help":
                         CommandHelp(cd);
+                        if (cd.Chat.ChatType == ChatType.Session) {
+                            handled = true;
+                        }
                         break;
                     case "server":
                     case "connect":
@@ -381,6 +386,10 @@ namespace Smuxi.Engine
                         CommandConfig(cd);
                         handled = true;
                         break;
+                    case "shutdown":
+                        CommandShutdown(cd);
+                        handled = true;
+                        break;
                 }
             } else {
                 // normal text
@@ -401,31 +410,28 @@ namespace Smuxi.Engine
             if (cd == null) {
                 throw new ArgumentNullException("cd");
             }
-            
-            MessageModel msg = new MessageModel();
-            TextMessagePartModel msgPart;
-            
-            msgPart = new TextMessagePartModel();
+            var builder = CreateMessageBuilder();
             // TRANSLATOR: this line is used as a label / category for a
             // list of commands below
-            msgPart.Text = "[" + _("Engine Commands") + "]";
-            msgPart.Bold = true;
-            msg.MessageParts.Add(msgPart);
-
-            cd.FrontendManager.AddMessageToChat(cd.Chat, msg);
+            builder.AppendHeader(_("Engine Commands"));
+            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
 
             string[] help = {
                 "help",
                 "connect/server protocol [protocol-parameters]",
-                "disconnect",
+                "disconnect [server]",
                 "network list",
-                "network close [server]",
-                "network switch [server]",
+                "network close [network]",
+                "network switch [network]",
                 "config (save|load)",
+                "shutdown"
             };
-            
-            foreach (string line in help) { 
-                cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " + line);
+
+            foreach (string line in help) {
+                builder = CreateMessageBuilder();
+                builder.AppendEventPrefix();
+                builder.AppendText(line);
+                cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
             }
         }
         
@@ -440,8 +446,15 @@ namespace Smuxi.Engine
             FrontendManager fm = cd.FrontendManager;
             
             string protocol;
-            if (cd.DataArray.Length >= 2) {
+            if (cd.DataArray.Length >= 3) {
                 protocol = cd.DataArray[1];
+            } else if (cd.DataArray.Length >= 2) {
+                // HACK: simply assume the user meant irc if not specified as
+                // Smuxi is still primarly an IRC client
+                protocol = "irc";
+                string cmd = String.Format("{0}connect irc {1}",
+                                           cd.CommandCharacter, cd.Parameter);
+                cd = new CommandModel(fm, cd.Chat, cd.CommandCharacter, cmd);
             } else {
                 _NotEnoughParameters(cd);
                 return;
@@ -528,14 +541,7 @@ namespace Smuxi.Engine
             IProtocolManager victim = null;
             if (cd.DataArray.Length >= 2) {
                 string server = cd.DataArray[1];
-                lock (_ProtocolManagers) {
-                    foreach (IProtocolManager nm in _ProtocolManagers) {
-                        if (nm.Host.ToLower() == server.ToLower()) {
-                            victim = nm;
-                            break;
-                        }
-                    }
-                }
+                victim = GetProtocolManagerByHost(server);
                 if (victim == null) {
                     fm.AddTextToChat(
                         cd.Chat,
@@ -616,6 +622,35 @@ namespace Smuxi.Engine
             }
         }
         
+        public void CommandShutdown(CommandModel cmd)
+        {
+            Trace.Call(cmd);
+
+            if (cmd == null) {
+                throw new ArgumentNullException("cmd");
+            }
+
+#if LOG4NET
+            f_Logger.Info("Shutting down...");
+#endif
+            lock (_ProtocolManagers) {
+                foreach (var protocolManager in _ProtocolManagers) {
+                    protocolManager.Disconnect(cmd.FrontendManager);
+                    protocolManager.Dispose();
+                }
+            }
+
+            if (IsLocal) {
+                // allow the frontend to cleanly terminate
+                return;
+            }
+
+#if LOG4NET
+            f_Logger.Debug("Terminating process...");
+#endif
+            Environment.Exit(0);
+        }
+
         public void CommandNetwork(CommandModel cd)
         {
             Trace.Call(cd);
@@ -653,8 +688,9 @@ namespace Smuxi.Engine
             lock (_ProtocolManagers) {
                 foreach (IProtocolManager nm in _ProtocolManagers) {
                     fm.AddTextToChat(cd.Chat, "-!- " +
-                        _("Type") + ": " + nm.Protocol + " " +
-                        _("Host") + ": " + nm.Host + " " + 
+                        _("Protocol") + ": " + nm.Protocol + " " +
+                        _("Network") + ": " + nm.NetworkID + " " +
+                        _("Host") + ": " + nm.Host + " " +
                         _("Port") + ": " + nm.Port);
                 }
             }
@@ -666,19 +702,12 @@ namespace Smuxi.Engine
             IProtocolManager pm = null;
             if (cd.DataArray.Length >= 3) {
                 // named protocol manager
-                string host = cd.DataArray[2].ToLower();
-                lock (_ProtocolManagers) {
-                    foreach (IProtocolManager protocolManager in _ProtocolManagers) {
-                        if (protocolManager.Host.ToLower() == host) {
-                            pm = protocolManager;
-                            break;
-                        }
-                    }
-                }
+                string network = cd.DataArray[2];
+                pm = GetProtocolManagerByNetwork(network);
                 if (pm == null) {
                     fm.AddTextToChat(cd.Chat, "-!- " +
-                        String.Format(_("Network close failed - could not find network with host: {0}"),
-                                      host));
+                        String.Format(_("Network close failed - could not find network: {0}"),
+                                      network));
                     return;
                 }
             } else if (cd.DataArray.Length >= 2) {
@@ -711,19 +740,17 @@ namespace Smuxi.Engine
             FrontendManager fm = cd.FrontendManager;
             if (cd.DataArray.Length >= 3) {
                 // named network manager
-                string host = cd.DataArray[2].ToLower();
-                lock (_ProtocolManagers) {
-                    foreach (IProtocolManager nm in _ProtocolManagers) {
-                        if (nm.Host.ToLower() == host) {
-                            fm.CurrentProtocolManager = nm;
-                            fm.UpdateNetworkStatus();
-                            return;
-                        }
-                    }
+                string network = cd.DataArray[2];
+                var pm = GetProtocolManagerByNetwork(network);
+                if (pm == null) {
+                    fm.AddTextToChat(cd.Chat, "-!- " +
+                        String.Format(
+                            _("Network switch failed - could not find network: {0}"),
+                                  network));
+                    return;
                 }
-                fm.AddTextToChat(cd.Chat, "-!- " +
-                    String.Format(_("Network switch failed - could not find network with host: {0}"),
-                                  host));
+                fm.CurrentProtocolManager = pm;
+                fm.UpdateNetworkStatus();
             } else if (cd.DataArray.Length >= 2) {
                 // next network manager
                 fm.NextProtocolManager();
@@ -765,7 +792,42 @@ namespace Smuxi.Engine
                 }
             }
         }
-        
+
+        public T CreateChat<T>(string id,
+                               string name,
+                               IProtocolManager protocolManager)
+                              where T : ChatModel
+        {
+            Trace.Call(id, name, protocolManager);
+
+            T chat;
+            Type chatType = typeof(T);
+            if (chatType == typeof(SessionChatModel)) {
+                chat = (T) Activator.CreateInstance(chatType, id, name);
+            } else if (chatType == typeof(PersonChatModel)) {
+                throw new NotSupportedException(
+                    "PersonModel is not supported, use " +
+                    "Session.CreatePersionChat() instead"
+                );
+            } else {
+                chat = (T) Activator.CreateInstance(chatType,
+                                                    id, name, protocolManager);
+            }
+            chat.ApplyConfig(UserConfig);
+            return chat;
+        }
+
+        public PersonChatModel CreatePersonChat(PersonModel person,
+                                                string id, string name,
+                                                IProtocolManager protocolManager)
+        {
+            Trace.Call(person, id, name, protocolManager);
+
+            var chat = new PersonChatModel(person, id, name, protocolManager);
+            chat.ApplyConfig(UserConfig);
+            return chat;
+        }
+
         public void AddChat(ChatModel chat)
         {
             Trace.Call(chat);
@@ -773,11 +835,17 @@ namespace Smuxi.Engine
             if (chat == null) {
                 throw new ArgumentNullException("chat");
             }
-            
+
+            chat.Position = GetSortedChatPosition(chat);
             lock (_Chats) {
                 _Chats.Add(chat);
+                if (chat.Position == -1) {
+                    chat.Position = _Chats.IndexOf(chat);
+                } else {
+                    MoveChat(chat, chat.Position);
+                }
             }
-            
+
             lock (_FrontendManagers) {
                 foreach (FrontendManager fm in _FrontendManagers.Values) {
                     fm.AddChat(chat);
@@ -798,8 +866,15 @@ namespace Smuxi.Engine
 #if LOG4NET
                     f_Logger.Warn("RemoveChat(): _Chats.Remove(" + chat + ") failed, ignoring...");
 #endif
+                    chat.Close();
                     return;
                 }
+                chat.Close();
+
+                // refresh chat positions
+                foreach (var schat in _Chats) {
+                    schat.Position = _Chats.IndexOf(schat);
+                }
             }
             
             lock (_FrontendManagers) {
@@ -856,6 +931,23 @@ namespace Smuxi.Engine
             }
         }
         
+        public void MoveChat(ChatModel chat, int newPosition)
+        {
+            Trace.Call(chat, newPosition);
+
+            if (chat == null) {
+                throw new ArgumentNullException("chat");
+            }
+
+            lock (_Chats) {
+                _Chats.Remove(chat);
+                _Chats.Insert(newPosition, chat);
+                foreach (var schat in _Chats) {
+                    schat.Position = _Chats.IndexOf(schat);
+                }
+            }
+        }
+
         public void AddTextToChat(ChatModel chat, string text)
         {
             AddTextToChat(chat, text, false);
@@ -894,14 +986,41 @@ namespace Smuxi.Engine
                 return;
             }
 
-            int buffer_lines = (int) UserConfig["Interface/Notebook/EngineBufferLines"];
-            if (buffer_lines > 0) {
-                chat.UnsafeMessages.Add(msg);
-                if (chat.UnsafeMessages.Count > buffer_lines) {
-                    chat.UnsafeMessages.RemoveAt(0);
+            lock (chat.MessageBuffer) {
+                try {
+                    chat.MessageBuffer.Add(msg);
+                } catch (Exception ex) {
+#if LOG4NET
+                    Trace.Call(chat, msg, ignoreFilters);
+                    f_Logger.Error(
+                        "AddMessageToChat(): " +
+                        "chat.MessageBuffer.Add() threw exception!", ex
+                    );
+#endif
+                    if (chat.MessageBuffer is Db4oMessageBuffer) {
+#if LOG4NET
+                        f_Logger.Error(
+                            "AddMessageToChat(): " +
+                            "Falling back to volatile message buffer..."
+                        );
+#endif
+                        chat.ResetMessageBuffer();
+                        chat.InitMessageBuffer(MessageBufferPersistencyType.Volatile);
+
+                        var builder = new MessageBuilder();
+                        builder.AppendEventPrefix();
+                        builder.AppendErrorText(
+                            _("Failed to write to chat history. " +
+                              "Your chat history will not be preserved. " +
+                              "Reason: {0}"),
+                            ex.Message
+                        );
+                        chat.MessageBuffer.Add(builder.ToMessage());
+
+                        chat.MessageBuffer.Add(msg);
+                    }
                 }
             }
-            
             lock (_FrontendManagers) {
                 foreach (FrontendManager fm in _FrontendManagers.Values) {
                     fm.AddMessageToChat(chat, msg);
@@ -1059,8 +1178,7 @@ namespace Smuxi.Engine
             if (!String.IsNullOrEmpty(server.Password)) {
                 password = server.Password;
             }
-            protocolManager.Connect(frontendManager, server.Hostname,
-                                    server.Port, server.Username, password);
+            protocolManager.Connect(frontendManager, server);
             if (protocolManager.Chat == null) {
                 // just in case the ProtocolManager is not setting the
                 // protocol chat
@@ -1256,6 +1374,172 @@ namespace Smuxi.Engine
             }
         }
 
+        IProtocolManager GetProtocolManagerByHost(string network)
+        {
+            lock (_ProtocolManagers) {
+                foreach (var manager in _ProtocolManagers) {
+                    if (String.Compare(manager.Host, network, true) == 0) {
+                        return manager;
+                    }
+                }
+            }
+            return null;
+        }
+
+        IProtocolManager GetProtocolManagerByNetwork(string network)
+        {
+            lock (_ProtocolManagers) {
+                foreach (var manager in _ProtocolManagers) {
+                    if (String.Compare(manager.NetworkID, network, true) == 0) {
+                        return manager;
+                    }
+                }
+            }
+            return null;
+        }
+
+        int GetSortedChatPosition(ChatModel chatModel)
+        {
+            int position = chatModel.Position;
+            if (position != -1) {
+                return position;
+            }
+
+            ChatType type = chatModel.ChatType;
+            if (type != ChatType.Person &&
+                type != ChatType.Group) {
+                return position;
+            }
+
+            // new group person and group chats behind their protocol chat
+            IProtocolManager pm = chatModel.ProtocolManager;
+            lock (_Chats) {
+                foreach (var chat in _Chats) {
+                    if (chat.ChatType == ChatType.Protocol &&
+                        chat.ProtocolManager == pm) {
+                        position = _Chats.IndexOf(chat) + 1;
+                        break;
+                    }
+                }
+
+                if (position == -1) {
+                    return position;
+                }
+
+                // now find the first chat with a different protocol manager
+                foreach (var chat in _Chats.Skip(position)) {
+                    if (chat.ProtocolManager != pm) {
+                        return _Chats.IndexOf(chat);
+                    }
+                }
+            }
+
+            // if there was no next protocol manager, simply append
+            // the chat way to the end
+            return -1;
+        }
+
+        void InitSessionChat()
+        {
+            _SessionChat = new SessionChatModel("smuxi", "Smuxi");
+            _Chats.Add(_SessionChat);
+
+            var builder = CreateMessageBuilder();
+            var text = builder.CreateText(_("Welcome to Smuxi"));
+            text.ForegroundColor = new TextColor(255, 0, 0);
+            text.Bold = true;
+            builder.AppendText(text);
+            builder.AppendText(Environment.NewLine);
+
+            text = builder.CreateText(
+                _("Type /help to get a list of available commands.")
+            );
+            text.Bold = true;
+            builder.AppendText(text);
+            builder.AppendText(Environment.NewLine);
+
+            text = builder.CreateText(_("After you have made a connection " +
+                "the list of available commands changes. Go to the newly " +
+                "opened connection tab and use the /help command again to " +
+                "see the extended command list."));
+            text.Bold = true;
+            builder.AppendText(text);
+            builder.AppendText(Environment.NewLine);
+
+            builder.AppendText(Environment.NewLine);
+
+            builder.AppendHeader("Smuxi News");
+            AddMessageToChat(_SessionChat,builder.ToMessage());
+        }
+
+        void UpdateNewsFeed()
+        {
+            Trace.Call();
+
+            try {
+                var url = "http://news.smuxi.org/feed.php";
+                var req = WebRequest.Create(url);
+                var proxySettings = new ProxySettings();
+                proxySettings.ApplyConfig(UserConfig);
+                req.Proxy = proxySettings.GetWebProxy(url);
+                if (req is HttpWebRequest) {
+                    var httpReq = (HttpWebRequest) req;
+                    httpReq.UserAgent = Engine.VersionString;
+                    if (NewsFeedLastModified != DateTime.MinValue) {
+                        httpReq.IfModifiedSince = NewsFeedLastModified;
+                    }
+                }
+                var res = req.GetResponse();
+                if (res is HttpWebResponse) {
+                    var httpRes = (HttpWebResponse) res;
+                    if (httpRes.StatusCode == HttpStatusCode.NotModified) {
+                        return;
+                    }
+                    NewsFeedLastModified = httpRes.LastModified;
+                }
+                var feed = AtomFeed.Load(res.GetResponseStream());
+                var sortedEntries = feed.Entry.OrderBy(x => x.Published);
+                foreach (var entry in sortedEntries) {
+                    if (SeenNewsFeedIds.Contains(entry.Id)) {
+                        continue;
+                    }
+                    SeenNewsFeedIds.Add(entry.Id);
+
+                    var msg = new FeedMessageBuilder();
+                    msg.Append(entry);
+                    if (!msg.IsEmpty) {
+                        msg.AppendText("\n");
+                        AddMessageToChat(SessionChat, msg.ToMessage());
+                    }
+                }
+            } catch (WebException ex) {
+                switch (ex.Status) {
+                    case WebExceptionStatus.ConnectFailure:
+                    case WebExceptionStatus.ConnectionClosed:
+                    case WebExceptionStatus.Timeout:
+                    case WebExceptionStatus.ReceiveFailure:
+                    case WebExceptionStatus.NameResolutionFailure:
+                    case WebExceptionStatus.ProxyNameResolutionFailure:
+#if LOG4NET
+                        f_Logger.Warn(
+                            String.Format(
+                                "UpdateNewsFeed(): Temporarily issue " +
+                                "detected, retrying in {0} min...",
+                                NewsFeedRetryInterval.Minutes
+                            ),
+                            ex
+                        );
+#endif
+                        NewsFeedTimer.Change(NewsFeedRetryInterval, NewsFeedUpdateInterval);
+                        break;
+                }
+            } catch (Exception ex) {
+#if LOG4NET
+                f_Logger.Error("UpdateNewsFeed(): Exception, ignored...", ex);
+#endif
+            }
+        }
+
         private static string _(string msg)
         {
             return LibraryCatalog.GetString(msg, _LibraryTextDomain);
