@@ -44,9 +44,11 @@ namespace Smuxi.Frontend.Gnome
         private StringCollection _History = new StringCollection();
         private int              _HistoryPosition;
         private bool             _HistoryChangedLine;
-        private Notebook         _Notebook;
         private CommandManager   _CommandManager;
-        
+        private new EntrySettings Settings { get; set; }
+
+        ChatViewManager ChatViewManager;
+
         /*
         public StringCollection History {
             get {
@@ -77,17 +79,18 @@ namespace Smuxi.Frontend.Gnome
         }
         */
 
-        public Entry(Notebook notebook)
+        public Entry(ChatViewManager chatViewManager)
         {
-            Trace.Call(notebook);
+            Trace.Call(chatViewManager);
 
-            if (notebook == null) {
-                throw new ArgumentNullException("notebook");
+            if (chatViewManager == null) {
+                throw new ArgumentNullException("chatViewManager");
             }
 
             _History.Add(String.Empty);
             
-            _Notebook = notebook;
+            ChatViewManager = chatViewManager;
+            Settings = new EntrySettings();
 
             InitCommandManager();
             Frontend.SessionPropertyChanged += delegate {
@@ -135,7 +138,7 @@ namespace Smuxi.Frontend.Gnome
              _Logger.Debug("added: '"+data+"' to history");
 #endif
 
-            if (_History.Count > (int)Frontend.UserConfig["Interface/Entry/CommandHistorySize"]) {
+            if (_History.Count > Settings.CommandHistorySize) {
                 _History.RemoveAt(0);
             } else {
                 _HistoryPosition += positiondiff;
@@ -225,20 +228,40 @@ namespace Smuxi.Frontend.Gnome
                 e.RetVal = true;
                 switch (key) {
                     case Gdk.Key.x:
-                        if (_Notebook.CurrentChatView.ChatModel.ChatType == ChatType.Session) {
+                    case Gdk.Key.X:
+                        if (ChatViewManager.CurrentChatView is SessionChatView) {
                             Frontend.FrontendManager.NextProtocolManager();
                         } else {
                             // don't break cut
                             e.RetVal = false;
                         }
                         break;
+                    case Gdk.Key.p:
+                    case Gdk.Key.P:
+                        ChatViewManager.CurrentChatNumber--;
+                        break;
+                    case Gdk.Key.n:
+                    case Gdk.Key.N:
+                        ChatViewManager.CurrentChatNumber++;
+                        break;
+                    case Gdk.Key.Tab:
+                    case Gdk.Key.ISO_Left_Tab:
+                        if ((e.Event.State & Gdk.ModifierType.ShiftMask) != 0) {
+                            ChatViewManager.CurrentChatNumber--;
+                        } else {
+                            ChatViewManager.CurrentChatNumber++;
+                        }
+                        break;
                     // don't break unicode input
                     case Gdk.Key.U:
                     // don't break copy/paste
                     case Gdk.Key.c:
+                    case Gdk.Key.C:
                     case Gdk.Key.v:
+                    case Gdk.Key.V:
                     // don't break select all
                     case Gdk.Key.a:
+                    case Gdk.Key.A:
                     // don't break jump one word left/right
                     case Gdk.Key.Right:
                     case Gdk.Key.Left:
@@ -247,10 +270,10 @@ namespace Smuxi.Frontend.Gnome
                         e.RetVal = false;
                         break;
                     case Gdk.Key.Home:
-                        _Notebook.CurrentChatView.ScrollToStart();
+                        ChatViewManager.CurrentChatView.ScrollToStart();
                         break;
                     case Gdk.Key.End:
-                        _Notebook.CurrentChatView.ScrollToEnd();
+                        ChatViewManager.CurrentChatView.ScrollToEnd();
                         break;
                 }
             }
@@ -305,9 +328,8 @@ namespace Smuxi.Frontend.Gnome
                         break;
                 }
 
-                if (pagenumber != -1 &&
-                    _Notebook.NPages >= pagenumber + 1) {
-                    _Notebook.Page = pagenumber;
+                if (pagenumber != -1) {
+                    ChatViewManager.CurrentChatNumber = pagenumber;
                 }
             }
 
@@ -325,7 +347,7 @@ namespace Smuxi.Frontend.Gnome
                     e.RetVal = true;
                     if (Frontend.MainWindow.CaretMode) {
                         // when we are in caret-mode change focus to output textview
-                        _Notebook.CurrentChatView.HasFocus = true;
+                        ChatViewManager.CurrentChatView.HasFocus = true;
                     } else {
                         if (Text.Length > 0) {
                             _NickCompletion();
@@ -345,10 +367,10 @@ namespace Smuxi.Frontend.Gnome
                     HistoryNext();
                     break;
                 case Gdk.Key.Page_Up:
-                    _Notebook.CurrentChatView.ScrollUp();
+                    ChatViewManager.CurrentChatView.ScrollUp();
                     break;
                 case Gdk.Key.Page_Down:
-                    _Notebook.CurrentChatView.ScrollDown();
+                    ChatViewManager.CurrentChatView.ScrollDown();
                     break;
             }
         }
@@ -378,7 +400,7 @@ namespace Smuxi.Frontend.Gnome
                     return true;
                 }
 
-                ChatView chat = Frontend.MainWindow.Notebook.CurrentChatView;
+                ChatView chat = ChatViewManager.CurrentChatView;
                 if (chat == null) {
                     return false;
                 }
@@ -462,11 +484,10 @@ namespace Smuxi.Frontend.Gnome
                 return;
             }
 
-            // BUG? REMOTING CALLs here?!? (would block GUI!)
             CommandModel cd = new CommandModel(
                 Frontend.FrontendManager,
-                _Notebook.CurrentChatView.ChatModel,
-                (string) Frontend.UserConfig["Interface/Entry/CommandCharacter"],
+                ChatViewManager.CurrentChatView.ChatModel,
+                Settings.CommandCharacter,
                 cmd
             );
 
@@ -507,6 +528,10 @@ namespace Smuxi.Frontend.Gnome
                         _CommandClear(cd);
                         handled = true;
                         break;
+                    case "list":
+                        _CommandList(cd);
+                        handled = true;
+                        break;
                 }
             }
             
@@ -515,18 +540,13 @@ namespace Smuxi.Frontend.Gnome
         
         private void _CommandHelp(CommandModel cd)
         {
-            MessageModel msg = new MessageModel();
-            TextMessagePartModel msgPart;
-            
-            msgPart = new TextMessagePartModel();
+            var chatView = ChatViewManager.GetChat(cd.Chat);
+            var builder = new MessageBuilder();
             // TRANSLATOR: this line is used as a label / category for a
             // list of commands below
-            msgPart.Text = "[" + _("Frontend Commands") + "]";
-            msgPart.Bold = true;
-            msg.MessageParts.Add(msgPart);
-            
-            cd.FrontendManager.AddMessageToChat(cd.Chat, msg);
-            
+            builder.AppendHeader(_("Frontend Commands"));
+            chatView.AddMessage(builder.ToMessage());
+
             string[] help = {
             "help",
             "window (number|channelname|queryname|close)",
@@ -534,16 +554,22 @@ namespace Smuxi.Frontend.Gnome
             "echo data",
             "exec command",
             "detach",
+            "list [search key]",
             };
-            
-            foreach (string line in help) { 
-                cd.FrontendManager.AddTextToChat(
-                    cd.Chat,
-                    String.Format("-!- {0}", line)
-                );
+
+            foreach (string line in help) {
+                builder = new MessageBuilder();
+                builder.AppendEventPrefix();
+                builder.AppendText(line);
+                chatView.AddMessage(builder.ToMessage());
             }
         }
-        
+
+        private void _CommandList(CommandModel cd)
+        {
+            Frontend.MainWindow.OpenFindGroupChatWindow(cd.Parameter);
+        }
+
         private void _CommandDetach(CommandModel cd)
         {
             Frontend.Quit();
@@ -584,62 +610,59 @@ namespace Smuxi.Frontend.Gnome
         {
             FrontendManager fm = cd.FrontendManager;
             if (cd.DataArray.Length >= 2) {
-                ChatModel currentChatModel = _Notebook.CurrentChatView.ChatModel;
-                string name;
-                if (cd.DataArray[1].ToLower() == "close") {
-                    cd.Chat.ProtocolManager.CloseChat(fm, cd.Chat);
+                var currentChat = ChatViewManager.CurrentChatView;
+                if (cd.Parameter.ToLower() == "close") {
+                    currentChat.Close();
                 } else {
-                    bool is_number = false;
-                    int pagecount = _Notebook.NPages;
                     try {
                         int number = Int32.Parse(cd.DataArray[1]);
-                        is_number = true;
-                        if (number <= pagecount) {
-                            _Notebook.CurrentPage = number - 1;
+                        if (number > ChatViewManager.Chats.Count) {
+                            return;
                         }
+                        ChatViewManager.CurrentChatNumber = number - 1;
+                        return;
                     } catch (FormatException) {
                     }
                     
-                    if (!is_number) {
-                        // seems to be query- or channelname
-                        // let's see if we find something
-                        ArrayList candidates = new ArrayList();
-                        for (int i = 0; i < pagecount; i++) {
-                            ChatView chatView = _Notebook.GetChat(i);
-                            ChatModel chatModel = chatView.ChatModel;
-                            
-                            if (chatModel.Name.ToLower() == cd.DataArray[1].ToLower()) {
-                                // name matches
-                                // first let's see if there is an exact match, if so, take it
-                                if ((chatModel.ChatType == currentChatModel.ChatType) &&
-                                    (chatModel.ProtocolManager == currentChatModel.ProtocolManager)) {
-                                    _Notebook.CurrentPage = i;
-                                    break;
-                                } else {
-                                    // there was no exact match
-                                    candidates.Add(i);
-                                }
-                            }
+                    // seems to be query- or channelname
+                    // let's see if we find something
+                    var seachKey = cd.Parameter.ToLower();
+                    var candidates = new List<ChatView>();
+                    foreach (var chatView in ChatViewManager.Chats) {
+
+                        if (chatView.Name.ToLower() != seachKey) {
+                            continue;
                         }
-                        
-                        if (candidates.Count > 0) {
-                            _Notebook.CurrentPage = (int)candidates[0];
+                        // name matches
+                        // let's see if there is an exact match, if so, take it
+                        if ((chatView.GetType() == currentChat.GetType()) &&
+                            (chatView.ProtocolManager == currentChat.ProtocolManager)) {
+                            candidates.Add(chatView);
+                            break;
+                        } else {
+                            // there was no exact match
+                            candidates.Add(chatView);
                         }
                     }
+
+                    if (candidates.Count == 0) {
+                        return;
+                    }
+                    ChatViewManager.CurrentChatView = candidates[0];
                 }
             }
         }
     
         private void _CommandClear(CommandModel cd)
         {
-            _Notebook.CurrentChatView.Clear();
+            ChatViewManager.CurrentChatView.Clear();
         }
 
         private void _NickCompletion()
         {
             // return if we don't support the current ChatView
-            if (!(_Notebook.CurrentChatView is GroupChatView) &&
-                !(_Notebook.CurrentChatView is PersonChatView)) {
+            if (!(ChatViewManager.CurrentChatView is GroupChatView) &&
+                !(ChatViewManager.CurrentChatView is PersonChatView)) {
                 return;
             }
 
@@ -690,10 +713,10 @@ namespace Smuxi.Frontend.Gnome
             bool partial_found = false;
             string nick = null;
 
-            ChatModel chat = _Notebook.CurrentChatView.ChatModel;
-            if (chat.ChatType == ChatType.Group) {
-                GroupChatModel cp = (GroupChatModel) chat;
-                if ((bool)Frontend.UserConfig["Interface/Entry/BashStyleCompletion"]) {
+            ChatView view = ChatViewManager.CurrentChatView;
+            if (view is GroupChatView) {
+                GroupChatView cp = (GroupChatView) view;
+                if (Settings.BashStyleCompletion) {
                     IList<string> result = cp.PersonLookupAll(word);
                     if (result == null || result.Count == 0) {
                         // no match
@@ -704,7 +727,7 @@ namespace Smuxi.Frontend.Gnome
                         string[] nickArray = new string[result.Count];
                         result.CopyTo(nickArray, 0);
                         string nicks = String.Join(" ", nickArray, 1, nickArray.Length - 1);
-                        _Notebook.CurrentChatView.AddMessage(
+                        ChatViewManager.CurrentChatView.AddMessage(
                             new MessageModel(String.Format("-!- {0}", nicks))
                         );
                         found = true;
@@ -719,17 +742,16 @@ namespace Smuxi.Frontend.Gnome
                      }
                 }
             } else {
-                PersonChatModel cp = (PersonChatModel) chat;
-                PersonModel person = cp.Person;
-
-                if (person.IdentityName.StartsWith(word, StringComparison.InvariantCultureIgnoreCase)) {
+                var personChat = (PersonChatView) ChatViewManager.CurrentChatView;
+                var personModel = personChat.PersonModel;
+                if (personModel.IdentityName.StartsWith(word,
+                        StringComparison.InvariantCultureIgnoreCase)) {
                     found = true;
-                    nick = cp.Person.IdentityName;
+                    nick = personModel.IdentityName;
                 }
             }
 
-            string completionChar = (string)
-                Frontend.UserConfig["Interface/Entry/CompletionCharacter"];
+            string completionChar = Settings.CompletionCharacter;
             // add leading @ back and supress completion character
             if (leadingAt) {
                 word = String.Format("@{0}", word);
@@ -802,6 +824,8 @@ namespace Smuxi.Frontend.Gnome
                 ModifyText(Gtk.StateType.Normal, theme.ForegroundColor.Value);
             }
             ModifyFont(theme.FontDescription);
+
+            Settings.ApplyConfig(config);
         }
 
         private void InitCommandManager()
