@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2008 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -28,8 +22,10 @@
 
 using System;
 using System.IO;
+using System.Linq;
 using System.Threading;
 using System.Reflection;
+using SysDiag = System.Diagnostics;
 using Smuxi.Engine;
 using Smuxi.Common;
 
@@ -63,6 +59,10 @@ namespace Smuxi.Frontend.Gnome
         private static bool               _InCrashHandler;
         private static bool               _InReconnectHandler;
 
+        public static string IconName { get; private set; }
+        public static bool HasSystemIconTheme { get; private set; }
+        public static bool HadSession { get; private set; }
+
         public static event EventHandler  SessionPropertyChanged;
 
         public static string Name {
@@ -117,6 +117,10 @@ namespace Smuxi.Frontend.Gnome
             set {
                 _Session = value;
 
+                if (value != null) {
+                    HadSession = true;
+                }
+
                 if (SessionPropertyChanged != null) {
                     SessionPropertyChanged(value, EventArgs.Empty);
                 }
@@ -125,7 +129,7 @@ namespace Smuxi.Frontend.Gnome
 
         public static bool IsLocalEngine {
             get {
-                return _Session == _LocalSession;
+                return _LocalSession != null && _Session == _LocalSession;
             }
         }
 
@@ -155,7 +159,16 @@ namespace Smuxi.Frontend.Gnome
                 return _FrontendConfig;
             }
         }
-        
+
+        public static bool UseLowBandwidthMode {
+            get {
+                if (_FrontendConfig == null) {
+                    return false;
+                }
+                return (bool) _FrontendConfig["UseLowBandwidthMode"];
+            }
+        }
+
         public static void Init(string[] args)
         {
             System.Threading.Thread.CurrentThread.Name = "Main";
@@ -171,46 +184,16 @@ namespace Smuxi.Frontend.Gnome
 #if LOG4NET
             _Logger.Info(_VersionString + " starting");
 #endif
-            
-#if GTK_SHARP_2_8 || GTK_SHARP_2_10
-            if (!GLib.Thread.Supported) {
-                GLib.Thread.Init();
-            }
-            _UIThreadID = System.Threading.Thread.CurrentThread.ManagedThreadId;
-#else
-            // with GTK# 2.8 we can do this better, see above
-            // GTK# 2.7.1 for MS .NET doesn't support that though.
-            if (Type.GetType("Mono.Runtime") == null) {
-                // when we don't run on Mono, we need to initialize glib ourself
-                GLib.Thread.Init();
-            }
-#endif
 
-            string appDir = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
-            string localeDir = Path.Combine(appDir, "locale");
-            if (!Directory.Exists(localeDir)) {
-                localeDir = Path.Combine(Defines.InstallPrefix, "share");
-                localeDir = Path.Combine(localeDir, "locale");
-            }
+            InitGtk(args);
 
-            LibraryCatalog.Init("smuxi-frontend-gnome", localeDir);
-#if LOG4NET
-            _Logger.Debug("Using locale data from: " + localeDir);
-#endif
-            
-            Gtk.Application.Init(Name, ref args);
-#if GTK_SHARP_2_10
-            GLib.ExceptionManager.UnhandledException += _OnUnhandledException;
-#endif
             //_SplashScreenWindow = new SplashScreenWindow();
 
             _FrontendConfig = new FrontendConfig(UIName);
             // loading and setting defaults
             _FrontendConfig.Load();
             _FrontendConfig.Save();
-            
-            Gtk.Window.DefaultIcon = new Gdk.Pixbuf(null, "icon.svg");
- 
+
             _MainWindow = new MainWindow();
 
             if (((string[]) FrontendConfig["Engines/Engines"]).Length == 0) {
@@ -253,20 +236,22 @@ namespace Smuxi.Frontend.Gnome
                 // HACK: SessionManager.Register() is not used for local engines
                 _LocalSession.RegisterFrontendUI(_MainWindow.UI);
             }
+
+            SyncConfig();
+
             _FrontendManager = _Session.GetFrontendManager(_MainWindow.UI);
             _FrontendManager.Sync();
-            
+
             // MS .NET doesn't like this with Remoting?
             if (Type.GetType("Mono.Runtime") != null) {
                 // when are running on Mono, all should be good
                 if (_UserConfig.IsCaching) {
                     // when our UserConfig is cached, we need to invalidate the cache
-                    _FrontendManager.ConfigChangedDelegate = new SimpleDelegate(_UserConfig.ClearCache);
+                    // DISABLED: see FrontendManager._OnConfigChanged
+                    //_FrontendManager.ConfigChangedDelegate = SyncConfig;
                 }
             }
-            
-            ApplyConfig(_UserConfig);
-            
+
             _MainWindow.ShowAll();
             // make sure entry got attention :-P
             _MainWindow.Entry.HasFocus = true;
@@ -292,6 +277,9 @@ namespace Smuxi.Frontend.Gnome
                 _FrontendManagerCheckerQueue.Queue(delegate {
                     // keep looping as long as the checker returns true
                     while (CheckFrontendManagerStatus()) {
+                        // FIXME: bail out somehow when we lost the connection
+                        // without an exception in the meantime
+
                         // only check once per minute
                         Thread.Sleep(60 * 1000);
                     }
@@ -302,45 +290,116 @@ namespace Smuxi.Frontend.Gnome
 #endif
                 });
             }
+            MainWindow.ChatViewManager.IsSensitive = true;
         }
         
         public static void DisconnectEngineFromGUI()
         {
-            Trace.Call();
+            DisconnectEngineFromGUI(true);
+        }
 
-            try {
-                // sync tab positions
-                if (!IsLocalEngine) {
-                    _MainWindow.Notebook.SyncPagePositions();
-                }
+        public static void DisconnectEngineFromGUI(bool cleanly)
+        {
+            Trace.Call(cleanly);
 
-                if (_FrontendManager != null) {
-                    _FrontendManager.IsFrontendDisconnecting = true;
+            MainWindow.ChatViewManager.IsSensitive = false;
+            if (cleanly) {
+                try {
+                    // sync tab positions
+                    if (!IsLocalEngine && !UseLowBandwidthMode) {
+                        _MainWindow.Notebook.SyncPagePositions();
+                    }
+
+                    if (_FrontendManager != null) {
+                        _FrontendManager.IsFrontendDisconnecting = true;
+                    }
+                    if (_Session != null) {
+                        _Session.DeregisterFrontendUI(_MainWindow.UI);
+                    }
+                } catch (System.Net.Sockets.SocketException) {
+                    // ignore as the connection is maybe already broken
+                } catch (System.Runtime.Remoting.RemotingException) {
+                    // ignore as the connection is maybe already broken
                 }
-                _Session.DeregisterFrontendUI(_MainWindow.UI);
-            } catch (System.Net.Sockets.SocketException ex) {
-                // ignore as the connection is maybe already broken
-            } catch (System.Runtime.Remoting.RemotingException ex) {
-                // ignore as the connection is maybe already broken
             }
-            _MainWindow.Hide();
-            _MainWindow.Notebook.RemoveAllPages();
+            if (_FrontendManagerCheckerQueue != null) {
+                _FrontendManagerCheckerQueue.Dispose();
+            }
+            _MainWindow.ChatViewManager.Clear();
+            _MainWindow.UpdateProgressBar();
             // make sure no stray SSH tunnel leaves behind
             _MainWindow.EngineManager.Disconnect();
-            
+            _MainWindow.NetworkStatus = null;
+            _MainWindow.Status = _("Disconnected from engine.");
+
             _FrontendManager = null;
             Session = null;
         }
 
         public static void ReconnectEngineToGUI()
         {
-            Trace.Call();
+            ReconnectEngineToGUI(true);
+        }
 
-            Frontend.DisconnectEngineFromGUI();
-            _MainWindow.EngineManager.Reconnect();
-            Session = _MainWindow.EngineManager.Session;
-            _UserConfig = _MainWindow.EngineManager.UserConfig;
-            Frontend.ConnectEngineToGUI();
+        public static void ReconnectEngineToGUI(bool cleanly)
+        {
+            Trace.Call(cleanly);
+
+            if (_InReconnectHandler) {
+#if LOG4NET
+                _Logger.Debug("ReconnectEngineToGUI(): already in reconnect " +
+                              "handler, ignoring reconnect...");
+#endif
+                return;
+            }
+
+            _InReconnectHandler = true;
+            var disconnectedEvent = new AutoResetEvent(false);
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    // delay the disconnect to give the reconnect some extra
+                    // time as NetworkManager is not accurate about when the
+                    // network is really ready
+                    GLib.Timeout.Add(5 * 1000, delegate {
+                        Frontend.DisconnectEngineFromGUI(cleanly);
+                        disconnectedEvent.Set();
+                        return false;
+                    });
+
+                    var successful = false;
+                    var attempt = 1;
+                    while (!successful) {
+                        Gtk.Application.Invoke(delegate {
+                            MainWindow.NetworkStatus = null;
+                            MainWindow.Status = String.Format(
+                                _("Reconnecting to engine... (attempt {0})"),
+                                attempt++
+                            );
+                        });
+                        try {
+                            disconnectedEvent.WaitOne();
+                            _MainWindow.EngineManager.Reconnect();
+                            successful = true;
+                        } catch (Exception ex) {
+#if LOG4NET
+                            _Logger.Debug("ReconnectEngineToGUI(): Exception", ex);
+#endif
+                            disconnectedEvent.Set();
+                            Thread.Sleep(30 * 1000);
+                        }
+                    }
+                    Session = _MainWindow.EngineManager.Session;
+                    _UserConfig = _MainWindow.EngineManager.UserConfig;
+
+                    Gtk.Application.Invoke(delegate {
+                        Frontend.ConnectEngineToGUI();
+                    });
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                } finally {
+                    _InReconnectHandler = false;
+                }
+            });
         }
 
         public static void Quit()
@@ -368,6 +427,21 @@ namespace Smuxi.Frontend.Gnome
             }
 
             if (_FrontendManager != null) {
+                if (IsLocalEngine) {
+                    try {
+                        var cmd = new CommandModel(
+                            _FrontendManager,
+                            MainWindow.ChatViewManager.CurrentChatView.ChatModel,
+                            null
+                        );
+                        Session.CommandShutdown(cmd);
+                    } catch (Exception ex) {
+#if LOG4NET
+                        _Logger.Error("Quit(): Exception", ex);
+#endif
+                    }
+                }
+
                 DisconnectEngineFromGUI();
             }
             
@@ -375,8 +449,8 @@ namespace Smuxi.Frontend.Gnome
             
             Environment.Exit(0);
         }
-        
-        private static bool IsGuiThread()
+
+        public static bool IsGuiThread()
         {
             return System.Threading.Thread.CurrentThread.ManagedThreadId == _UIThreadID;
         }
@@ -462,47 +536,8 @@ namespace Smuxi.Frontend.Gnome
                     // one reconnect is good enough and a crash we won't survive
                     return;
                 }
-                _InReconnectHandler = true;
 
-                Gtk.MessageDialog md = new Gtk.MessageDialog(parent,
-                    Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
-                    Gtk.ButtonsType.OkCancel, _("The frontend has lost the connection to the server.\nDo you want to reconnect now?"));
-                Gtk.ResponseType res = (Gtk.ResponseType) md.Run();
-                md.Destroy();
-                
-                if (res == Gtk.ResponseType.Ok) {
-                    while (true) {
-                        try {
-                            Frontend.ReconnectEngineToGUI();
-                            // yay, we made it
-                            _InReconnectHandler = false;
-                            break;
-                        } catch (Exception e) {
-#if LOG4NET
-                             _Logger.Error("ShowException(): Reconnect failed, exception:", e);
-#endif
-                            var msg = _("Reconnecting to the server has failed.\nDo you want to try again?");
-                            // the parent window is hidden (MainWindow) at this
-                            // point thus modal doesn't make sense here
-                            md = new Gtk.MessageDialog(parent,
-                                Gtk.DialogFlags.DestroyWithParent,
-                                Gtk.MessageType.Error,
-                                Gtk.ButtonsType.OkCancel, msg);
-                            md.SetPosition(Gtk.WindowPosition.CenterAlways);
-                            res = (Gtk.ResponseType) md.Run();
-                            md.Destroy();
-
-                            if (res != Gtk.ResponseType.Ok) {
-                                // give up
-                                Quit();
-                                return;
-                            }
-                        }
-                    }
-                    return;
-                }
-                
-                Quit();
+                Frontend.ReconnectEngineToGUI();
                 return;
             }
 
@@ -516,7 +551,14 @@ namespace Smuxi.Frontend.Gnome
             CrashDialog cd = new CrashDialog(parent, ex);
             cd.Run();
             cd.Destroy();
-            
+
+            if (SysDiag.Debugger.IsAttached) {
+                // allow the debugger to examine the situation
+                //SysDiag.Debugger.Break();
+                // HACK: Break() would be nicer but crashes the runtime
+                throw ex;
+            }
+
             Quit();
         }
         
@@ -535,7 +577,53 @@ namespace Smuxi.Frontend.Gnome
             diag.Run();
             diag.Destroy();
         }
-        
+
+        public static bool ShowReconnectDialog(Gtk.Window parent)
+        {
+            Trace.Call(parent);
+
+            Gtk.MessageDialog md = new Gtk.MessageDialog(parent,
+                Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
+                Gtk.ButtonsType.OkCancel, _("The frontend has lost the connection to the server.\nDo you want to reconnect now?"));
+            Gtk.ResponseType res = (Gtk.ResponseType) md.Run();
+            md.Destroy();
+
+            if (res != Gtk.ResponseType.Ok) {
+                Quit();
+                return false;
+            }
+
+            while (true) {
+                try {
+                    Frontend.ReconnectEngineToGUI();
+                    // yay, we made it
+                    _InReconnectHandler = false;
+                    break;
+                } catch (Exception e) {
+#if LOG4NET
+                     _Logger.Error("ShowReconnectDialog(): Reconnect failed, exception:", e);
+#endif
+                    var msg = _("Reconnecting to the server has failed.\nDo you want to try again?");
+                    // the parent window is hidden (MainWindow) at this
+                    // point thus modal doesn't make sense here
+                    md = new Gtk.MessageDialog(parent,
+                        Gtk.DialogFlags.DestroyWithParent,
+                        Gtk.MessageType.Error,
+                        Gtk.ButtonsType.OkCancel, msg);
+                    md.SetPosition(Gtk.WindowPosition.CenterAlways);
+                    res = (Gtk.ResponseType) md.Run();
+                    md.Destroy();
+
+                    if (res != Gtk.ResponseType.Ok) {
+                        // give up
+                        Quit();
+                        return false;
+                    }
+                }
+            }
+            return true;
+        }
+
         public static void ApplyConfig(UserConfig userConfig)
         {
             Trace.Call(userConfig);
@@ -547,16 +635,52 @@ namespace Smuxi.Frontend.Gnome
             _MainWindow.ApplyConfig(userConfig);
         }
 
+        public static Gdk.Pixbuf LoadIcon(string iconName, int size,
+                                          string resourceName)
+        {
+            Trace.Call(iconName, size, resourceName);
+
+            if (iconName == null) {
+                throw new ArgumentNullException("iconName");
+            }
+            if (resourceName == null) {
+                throw new ArgumentNullException("resourceName");
+            }
+
+            try {
+                // we use this method from type initializers so it has to deal
+                // with GDK/GTK thread locking
+                Gdk.Threads.Enter();
+                var theme = Gtk.IconTheme.Default;
+                if (!theme.HasIcon(iconName) ||
+                    !theme.GetIconSizes(iconName).Contains(size)) {
+                    Gtk.IconTheme.AddBuiltinIcon(
+                        iconName,
+                        size,
+                        new Gdk.Pixbuf(null, resourceName, size, size)
+                    );
+#if LOG4NET
+                    _Logger.DebugFormat(
+                        "LoadIcon(): Added '{0}' to built-in icon theme",
+                        resourceName
+                    );
+#endif
+                }
+                return theme.LoadIcon(iconName, size,
+                                      Gtk.IconLookupFlags.UseBuiltin);
+            } finally {
+                Gdk.Threads.Leave();
+            }
+        }
+
 #if GTK_SHARP_2_10
         private static void _OnUnhandledException(GLib.UnhandledExceptionArgs e)
         {
-            Trace.Call(e);
+            Trace.CallFull(e);
             
             lock (_UnhandledExceptionSyncRoot) {
                 if (e.ExceptionObject is Exception) {
                     ShowException((Exception) e.ExceptionObject);
-                    
-                    Quit();
                 }
             }
         }
@@ -597,7 +721,121 @@ namespace Smuxi.Frontend.Gnome
 
             return false;
         }
-        
+
+        private static void InitGtk(string[] args)
+        {
+            if (Environment.OSVersion.Platform == PlatformID.Win32NT) {
+                InitGtkPathWin();
+            }
+
+#if GTK_SHARP_2_8 || GTK_SHARP_2_10
+            if (!GLib.Thread.Supported) {
+                GLib.Thread.Init();
+            }
+#else
+            // with GTK# 2.8 we can do this better, see above
+            // GTK# 2.7.1 for MS .NET doesn't support that though.
+            if (Type.GetType("Mono.Runtime") == null) {
+                // when we don't run on Mono, we need to initialize glib ourself
+                GLib.Thread.Init();
+            }
+#endif
+            _UIThreadID = System.Threading.Thread.CurrentThread.ManagedThreadId;
+
+            string appDir = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
+            string localeDir = Path.Combine(appDir, "locale");
+            if (!Directory.Exists(localeDir)) {
+                localeDir = Path.Combine(Defines.InstallPrefix, "share");
+                localeDir = Path.Combine(localeDir, "locale");
+            }
+
+            LibraryCatalog.Init("smuxi-frontend-gnome", localeDir);
+#if LOG4NET
+            _Logger.Debug("InitGtk(): Using locale data from: " + localeDir);
+#endif
+            Gtk.Application.Init(Name, ref args);
+#if GTK_SHARP_2_10
+            GLib.ExceptionManager.UnhandledException += _OnUnhandledException;
+#endif
+
+            IconName = "smuxi-frontend-gnome";
+            var iconPath = Path.Combine(Defines.InstallPrefix, "share");
+            iconPath = Path.Combine(iconPath, "icons");
+            var theme = Gtk.IconTheme.Default;
+            var iconInfo = theme.LookupIcon(IconName, -1, 0);
+            HasSystemIconTheme = iconInfo != null &&
+                                 iconInfo.Filename != null &&
+                                 iconInfo.Filename.StartsWith(iconPath);
+#if LOG4NET
+            _Logger.DebugFormat("InitGtk(): Using {0} icon theme",
+                                HasSystemIconTheme ? "system" : "built-in");
+#endif
+
+            if (HasSystemIconTheme) {
+                Gtk.Window.DefaultIconName = "smuxi-frontend-gnome";
+            } else {
+                Gtk.Window.DefaultIcon = Frontend.LoadIcon(
+                    "smuxi-frontend-gnome", 256, "icon_256x256.png"
+                );
+            }
+        }
+
+        private static void InitGtkPathWin()
+        {
+            // HACK: Force GTK# to use the right GTK+ install as the PATH
+            // environment variable might contain other GTK+ installs
+            var installPath = (string) Microsoft.Win32.Registry.GetValue(
+                "HKEY_LOCAL_MACHINE\\SOFTWARE\\Novell\\GtkSharp\\InstallFolder",
+                "", null
+            );
+            if (installPath == null) {
+#if LOG4NET
+                _Logger.Error("InitGtkPathWin(): couldn't obtain GTK# installation folder from registry. GTK# is probably incorrectly installed!");
+                return;
+#endif
+            }
+
+            var binPath = Path.Combine(installPath, "bin");
+            var currentPath = Environment.GetEnvironmentVariable("PATH");
+            var newPath = String.Format("{0}{1}{2}", binPath, Path.PathSeparator, currentPath);
+#if LOG4NET
+            _Logger.Debug("InitGtkPathWin(): current PATH: " + currentPath);
+            _Logger.Debug("InitGtkPathWin(): new PATH: " + newPath);
+#endif
+            Environment.SetEnvironmentVariable("PATH", newPath);
+        }
+
+        static void SyncConfig()
+        {
+            Trace.Call();
+
+            if (EngineVersion >= new Version("0.8.1.1")) {
+                var config = UserConfig;
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        config.SyncCache();
+                    } catch (Exception ex) {
+#if LOG4NET
+                        _Logger.Error("SyncConfig(): " +
+                                      "Exception during config sync", ex);
+#endif
+                    } finally {
+                        Gtk.Application.Invoke(delegate {
+                            ApplyConfig(config);
+                        });
+                    }
+                });
+            } else {
+                if (!IsGuiThread()) {
+                    Gtk.Application.Invoke(delegate {
+                        SyncConfig();
+                    });
+                    return;
+                }
+                ApplyConfig(UserConfig);
+            }
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
