@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2010 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2010-2011 Mirco Bauer <meebey@meebey.net>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 // 
@@ -43,6 +43,8 @@ namespace Smuxi.Frontend.Gnome
         ChatViewManager ChatViewManager { get; set; }
         Dictionary<ChatView, Indicator> Indicators { get; set; }
         Dictionary<ChatView, MessageTextViewMessageHighlightedEventHandler> HighlightEventHandlers { get; set; }
+        bool IsInitialized { get; set; }
+        bool IsEnabled { get; set; }
 
         static IndicateManager()
         {
@@ -73,7 +75,13 @@ namespace Smuxi.Frontend.Gnome
                 <ChatView,
                  MessageTextViewMessageHighlightedEventHandler>();
 
-            Init();
+            try {
+                Init();
+            } catch (Exception ex) {
+#if LOG4NET
+                Logger.Error("IndicateManager(): initialization failed: ", ex);
+#endif
+            }
         }
         
         public void Dispose()
@@ -98,10 +106,18 @@ namespace Smuxi.Frontend.Gnome
                 throw new ArgumentNullException("userConfig");
             }
 
+            if (!IsInitialized) {
+                return;
+            }
+
             if ((bool) userConfig["Interface/Notification/MessagingMenuEnabled"]) {
                 Server.Show();
+                IsEnabled = true;
+                // hide the main window instead of closing it
+                MainWindow.NotificationAreaIconMode = NotificationAreaIconMode.Closed;
             } else {
                 Server.Hide();
+                IsEnabled = false;
             }
         }
 
@@ -125,9 +141,24 @@ namespace Smuxi.Frontend.Gnome
             }
             */
 
-            var desktopFile = Path.Combine(Defines.InstallPrefix, "share");
-            desktopFile = Path.Combine(desktopFile, "applications");
-            desktopFile = Path.Combine(desktopFile, "smuxi-frontend-gnome.desktop");
+            var partialPath = "share";
+            partialPath = Path.Combine(partialPath, "applications");
+            partialPath = Path.Combine(partialPath, "smuxi-frontend-gnome.desktop");
+
+            var insDesktopFile = Path.Combine(Defines.InstallPrefix, partialPath);
+            var sysDesktopFile = Path.Combine("/usr", partialPath);
+            string desktopFile = null;
+            if (File.Exists(insDesktopFile)) {
+                desktopFile = insDesktopFile;
+            } else if (File.Exists(sysDesktopFile)) {
+                desktopFile = sysDesktopFile;
+            } else {
+#if LOG4NET
+                Logger.Error("Init(): smuxi-frontend-gnome.desktop could not " +
+                             " be found, thus no messaging menu :/");
+#endif
+                return;
+            }
 
             Server.SetType("message.im");
             Server.DesktopFile(desktopFile);
@@ -138,21 +169,33 @@ namespace Smuxi.Frontend.Gnome
 
             ChatViewManager.ChatAdded   += OnChatViewManagerChatAdded;
             ChatViewManager.ChatRemoved += OnChatViewManagerChatRemoved;
+
+            IsInitialized = true;
         }
 
         void OnMainWindowFocusInEvent(object sender, Gtk.FocusInEventArgs e)
         {
-            Trace.Call(sender, e);
+            if (MainWindow.Notebook.IsBrowseModeEnabled) {
+                return;
+            }
 
             var currentChatView = MainWindow.Notebook.CurrentChatView;
+            if (currentChatView == null) {
+                return;
+            }
             DisposeIndicator(currentChatView);
         }
 
         void OnMainWindowNotebookSwitchPage(object sender, Gtk.SwitchPageArgs e)
         {
-            Trace.Call(sender, e);
+            if (MainWindow.Notebook.IsBrowseModeEnabled) {
+                return;
+            }
 
             var currentChatView = MainWindow.Notebook.CurrentChatView;
+            if (currentChatView == null) {
+                return;
+            }
             DisposeIndicator(currentChatView);
         }
 
@@ -160,9 +203,7 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(sender, e);
 
-            MainWindow.PresentWithTime(
-                (uint) (DateTime.UtcNow - UnixEpoch).TotalSeconds
-            );
+            MainWindow.PresentWithTime(e.Timestamp);
         }
 
         void OnChatViewManagerChatAdded(object sender, ChatViewManagerChatAddedEventArgs e)
@@ -192,15 +233,18 @@ namespace Smuxi.Frontend.Gnome
             }
 
             e.ChatView.OutputMessageTextView.MessageHighlighted -= handler;
+
+            // close possibly active indicator
+            DisposeIndicator(e.ChatView);
         }
 
         void OnChatViewMessageHighlighted(object sender,
                                           MessageTextViewMessageHighlightedEventArgs e,
                                           ChatView chatView)
         {
-            Trace.Call(sender, e, chatView);
-
-            if (MainWindow.HasToplevelFocus) {
+            if (!IsEnabled ||
+                e.Message.TimeStamp <= chatView.SyncedLastSeenHighlight ||
+                MainWindow.HasToplevelFocus) {
                 return;
             }
 
@@ -241,9 +285,7 @@ namespace Smuxi.Frontend.Gnome
             indicator.SetPropertyBool("draw-attention", true);
             indicator.UserDisplay += delegate {
                 try {
-                    MainWindow.PresentWithTime(
-                        (uint) (DateTime.UtcNow - UnixEpoch).TotalSeconds
-                    );
+                    MainWindow.Present();
                     MainWindow.Notebook.CurrentChatView = chatView;
                     DisposeIndicator(chatView);
                 } catch (Exception ex) {
@@ -267,15 +309,25 @@ namespace Smuxi.Frontend.Gnome
 
         void DisposeIndicator(ChatView chatView)
         {
-            Trace.Call(chatView);
-
             Indicator indicator;
             if (!Indicators.TryGetValue(chatView, out indicator)) {
                 return;
             }
+#if LOG4NET
+            Logger.Debug("DisposeIndicator(): disposing indicator for: " +
+                         chatView.Name);
+#endif
 
-            indicator.Hide();
-            Indicators.Remove(chatView);
+            try {
+                indicator.Hide();
+            } catch (Exception ex) {
+#if LOG4NET
+                Logger.Error("DisposeIndicator(): " +
+                             "indicator.Hide() thew exception", ex);
+#endif
+            } finally {
+                Indicators.Remove(chatView);
+            }
         }
 
         string GetNick(MessageModel msg)
