@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2008 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -53,8 +47,10 @@ namespace Smuxi.Frontend.Gnome
         private ChatViewManager  _ChatViewManager;
         private IFrontendUI      _UI;
         private EngineManager    _EngineManager;
+        private Gtk.ImageMenuItem _OpenChatMenuItem;
         private Gtk.MenuItem     _CloseChatMenuItem;
         private Gtk.ImageMenuItem _OpenLogChatMenuItem;
+        private Gtk.ImageMenuItem _FindGroupChatMenuItem;
         private NotificationAreaIconMode _NotificationAreaIconMode;
 #if GTK_SHARP_2_10
         private StatusIconManager _StatusIconManager;
@@ -64,11 +60,20 @@ namespace Smuxi.Frontend.Gnome
 #endif
 #if NOTIFY_SHARP
         private NotifyManager    _NotifyManager;
+#endif
+#if IPC_DBUS
+        private NetworkManager   _NetworkManager;
 #endif
         private bool             _IsMinimized;
         private bool             _IsMaximized;
         private bool             _IsFullscreen;
-        
+
+        public Gtk.MenuBar MenuBar {
+            get {
+                return _MenuBar;
+            }
+        }
+
         public bool ShowMenuBar {
             get {
                 return _MenuBar.Visible;
@@ -108,17 +113,25 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
-        public new Gtk.Statusbar NetworkStatusbar {
-            get {
-                return _NetworkStatusbar;
+        public string NetworkStatus {
+            set {
+                if (value == null) {
+                    value = String.Empty;
+                }
+                _NetworkStatusbar.Pop(0);
+                _NetworkStatusbar.Push(0, value);
             }
         } 
 
-        public new Gtk.Statusbar Statusbar {
-            get {
-                return _Statusbar;
+        public string Status {
+            set {
+                if (value == null) {
+                    value = String.Empty;
+                }
+                _Statusbar.Pop(0);
+                _Statusbar.Push(0, value);
             }
-        } 
+        }
 
         public Gtk.ProgressBar ProgressBar {
             get {
@@ -158,6 +171,15 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public NotificationAreaIconMode NotificationAreaIconMode {
+            get {
+                return _NotificationAreaIconMode;
+            }
+            set {
+                _NotificationAreaIconMode = value;
+            }
+        }
+
         public EventHandler Minimized;
         public EventHandler Unminimized;
 
@@ -175,6 +197,10 @@ namespace Smuxi.Frontend.Gnome
             } else {
                 heigth = 600;
             }
+            if (width < -1 || heigth < -1) {
+                width = -1;
+                heigth = -1;
+            }
             if (width == -1 && heigth == -1) {
                 SetDefaultSize(800, 600);
                 Maximize();
@@ -196,12 +222,16 @@ namespace Smuxi.Frontend.Gnome
             } else {
                 y = 0;
             }
+            if (x < 0 || y < 0) {
+                x = 0;
+                y = 0;
+            }
             if (x == 0 && y == 0) {
                 SetPosition(Gtk.WindowPosition.Center);
             } else {
                 Move(x, y);
             }
-            
+
             DeleteEvent += OnDeleteEvent;
             FocusInEvent += OnFocusInEvent;
             FocusOutEvent += OnFocusOutEvent;
@@ -225,12 +255,20 @@ namespace Smuxi.Frontend.Gnome
 
             item = new Gtk.ImageMenuItem(Gtk.Stock.Preferences, agrp);
             item.Activated += new EventHandler(_OnPreferencesButtonClicked);
+            item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(item);
             
             menu.Append(new Gtk.SeparatorMenuItem());
             
             item = new Gtk.ImageMenuItem(Gtk.Stock.Quit, agrp);
             item.Activated += new EventHandler(_OnQuitButtonClicked);
+            item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(item);
             
             // Menu - Server
@@ -261,15 +299,17 @@ namespace Smuxi.Frontend.Gnome
             item.Submenu = menu;
             _MenuBar.Append(item);
             
-            image_item = new Gtk.ImageMenuItem(_("Open / Join Chat"));
-            image_item.Image = new Gtk.Image(Gtk.Stock.Open, Gtk.IconSize.Menu);
-            image_item.Activated += OnChatOpenChatButtonClicked;
-            menu.Append(image_item);
+            _OpenChatMenuItem = new Gtk.ImageMenuItem(_("Open / Join Chat"));
+            _OpenChatMenuItem.Image = new Gtk.Image(Gtk.Stock.Open, Gtk.IconSize.Menu);
+            _OpenChatMenuItem.Activated += OnChatOpenChatButtonClicked;
+            _OpenChatMenuItem.Sensitive = false;
+            menu.Append(_OpenChatMenuItem);
                     
-            image_item = new Gtk.ImageMenuItem(_("_Find Group Chat"));
-            image_item.Image = new Gtk.Image(Gtk.Stock.Find, Gtk.IconSize.Menu);
-            image_item.Activated += OnChatFindGroupChatButtonClicked;
-            menu.Append(image_item);
+            _FindGroupChatMenuItem = new Gtk.ImageMenuItem(_("_Find Group Chat"));
+            _FindGroupChatMenuItem.Image = new Gtk.Image(Gtk.Stock.Find, Gtk.IconSize.Menu);
+            _FindGroupChatMenuItem.Activated += OnChatFindGroupChatButtonClicked;
+            _FindGroupChatMenuItem.Sensitive = false;
+            menu.Append(_FindGroupChatMenuItem);
             
             image_item = new Gtk.ImageMenuItem(_("C_lear All Activity"));
             image_item.Image = new Gtk.Image(Gtk.Stock.Clear, Gtk.IconSize.Menu);
@@ -286,6 +326,10 @@ namespace Smuxi.Frontend.Gnome
             akey.AccelMods = Gdk.ModifierType.ControlMask;
             akey.Key = Gdk.Key.Page_Down;
             image_item.AddAccelerator("activate", agrp, akey);
+            image_item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(image_item);
             
             image_item = new Gtk.ImageMenuItem(_("_Previous Chat"));
@@ -296,6 +340,10 @@ namespace Smuxi.Frontend.Gnome
             akey.AccelMods = Gdk.ModifierType.ControlMask;
             akey.Key = Gdk.Key.Page_Up;
             image_item.AddAccelerator("activate", agrp, akey);
+            image_item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(image_item);
             
             menu.Append(new Gtk.SeparatorMenuItem());
@@ -348,6 +396,10 @@ namespace Smuxi.Frontend.Gnome
 
             _CloseChatMenuItem = new Gtk.ImageMenuItem(Gtk.Stock.Close, agrp);
             _CloseChatMenuItem.Activated += OnCloseChatMenuItemActivated;
+            _CloseChatMenuItem.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(_CloseChatMenuItem);
 
             // Menu - Engine
@@ -384,6 +436,10 @@ namespace Smuxi.Frontend.Gnome
             akey.AccelFlags = Gtk.AccelFlags.Visible;
             akey.Key = Gdk.Key.F7;
             item.AddAccelerator("activate", agrp, akey);
+            item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(item);
             
             item = new Gtk.CheckMenuItem(_("_Browse Mode"));
@@ -398,6 +454,10 @@ namespace Smuxi.Frontend.Gnome
             akey.AccelFlags = Gtk.AccelFlags.Visible;
             akey.Key = Gdk.Key.F8;
             item.AddAccelerator("activate", agrp, akey);
+            item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(item);
 
             _ShowMenuBarItem = new Gtk.CheckMenuItem(_("Show _Menubar"));
@@ -423,6 +483,10 @@ namespace Smuxi.Frontend.Gnome
             akey.AccelFlags = Gtk.AccelFlags.Visible;
             akey.Key = Gdk.Key.F11;
             item.AddAccelerator("activate", agrp, akey);
+            item.AccelCanActivate += delegate(object o, Gtk.AccelCanActivateArgs args) {
+                // allow the accelerator to be used even when the menu bar is hidden
+                args.RetVal = true;
+            };
             menu.Append(item);
 
             // Menu - Help
@@ -445,6 +509,7 @@ namespace Smuxi.Frontend.Gnome
             _ChatViewManager.LoadAll(System.IO.Path.GetDirectoryName(asm.Location),
                                      "smuxi-frontend-gnome-*.dll");
             _ChatViewManager.ChatAdded += OnChatViewManagerChatAdded;
+            _ChatViewManager.ChatSynced += OnChatViewManagerChatSynced;
             _ChatViewManager.ChatRemoved += OnChatViewManagerChatRemoved;
             
 #if GTK_SHARP_2_10
@@ -456,16 +521,20 @@ namespace Smuxi.Frontend.Gnome
 #if NOTIFY_SHARP
             _NotifyManager = new NotifyManager(this, _ChatViewManager);
 #endif
+#if IPC_DBUS
+            _NetworkManager = new NetworkManager(_ChatViewManager);
+#endif
 
             _UI = new GnomeUI(_ChatViewManager);
             
             // HACK: Frontend.FrontendConfig out of scope
             _EngineManager = new EngineManager(Frontend.FrontendConfig, _UI);
 
-            _Entry = new Entry(_Notebook);
+            _Entry = new Entry(_ChatViewManager);
             
             _ProgressBar = new Gtk.ProgressBar();
-            
+            _ProgressBar.BarStyle = Gtk.ProgressBarStyle.Continuous;
+
             Gtk.VBox vbox = new Gtk.VBox();
             vbox.PackStart(_MenuBar, false, false, 0);
             vbox.PackStart(_Notebook, true, true, 0);
@@ -593,13 +662,17 @@ namespace Smuxi.Frontend.Gnome
             
             try {
                 UrgencyHint = false;
+                if (_Notebook.IsBrowseModeEnabled) {
+                    return;
+                }
+
                 ChatView chatView = _Notebook.CurrentChatView;
                 if (chatView != null) {
                     // clear activity and highlight
                     chatView.HasHighlight = false;
                     chatView.HasActivity = false;
                     var lastMsg = chatView.OutputMessageTextView.LastMessage;
-                    if (lastMsg == null) {
+                    if (lastMsg == null || Frontend.UseLowBandwidthMode) {
                         return;
                     }
                     // update last seen highlight
@@ -624,6 +697,10 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
 
             try {
+                if (_Notebook.IsBrowseModeEnabled) {
+                    return;
+                }
+
                 var chatView = _Notebook.CurrentChatView;
                 if (chatView == null) {
                     return;
@@ -717,15 +794,24 @@ namespace Smuxi.Frontend.Gnome
             try {
                 OpenChatDialog dialog = new OpenChatDialog(this);
                 int res = dialog.Run();
-                
-                IProtocolManager manager = Frontend.FrontendManager.CurrentProtocolManager;
+
+                var chatView = Notebook.CurrentChatView;
+                if (chatView == null) {
+                    return;
+                }
+
+                // FIXME: REMOTING CALL
+                var manager = chatView.ChatModel.ProtocolManager;
+                if (manager == null) {
+                    return;
+                }
                 ChatModel chat;
                 switch (dialog.ChatType) {
                     case ChatType.Group:
                         chat = new GroupChatModel(
                             dialog.ChatName, 
                             dialog.ChatName,
-                            manager
+                            null
                         );
                         break;
                     case ChatType.Person:
@@ -733,7 +819,7 @@ namespace Smuxi.Frontend.Gnome
                             null,
                             dialog.ChatName, 
                             dialog.ChatName,
-                            manager
+                            null
                         );
                         break;
                     default:
@@ -750,7 +836,13 @@ namespace Smuxi.Frontend.Gnome
                     return;
                 }
                 
-                manager.OpenChat(Frontend.FrontendManager, chat);
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        manager.OpenChat(Frontend.FrontendManager, chat);
+                    } catch (Exception ex) {
+                        Frontend.ShowException(this, ex);
+                    }
+                });
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -759,24 +851,53 @@ namespace Smuxi.Frontend.Gnome
         protected virtual void OnChatFindGroupChatButtonClicked(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
-            
             try {
-                IProtocolManager manager = Frontend.FrontendManager.CurrentProtocolManager;
-                FindGroupChatDialog dialog = new FindGroupChatDialog(
-                    this, manager
-                );
-                int res = dialog.Run();
-                GroupChatModel groupChat = dialog.GroupChat;
-                dialog.Destroy();
-                if (res != (int) Gtk.ResponseType.Ok) {
-                    return;
-                }
-                
-                manager.OpenChat(Frontend.FrontendManager, groupChat);
+                OpenFindGroupChatWindow();
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
         }
+
+        public void OpenFindGroupChatWindow()
+        {
+            OpenFindGroupChatWindow(null);
+        }
+
+        public void OpenFindGroupChatWindow(string searchKey)
+        {
+            var chatView = Notebook.CurrentChatView;
+            if (chatView == null) {
+                return;
+            }
+
+            // FIXME: REMOTING CALL
+            var manager = chatView.ChatModel.ProtocolManager;
+            if (manager == null) {
+                return;
+            }
+
+            FindGroupChatDialog dialog = new FindGroupChatDialog(
+                this, manager
+            );
+            if (!String.IsNullOrEmpty(searchKey)) {
+                dialog.NameEntry.Text = searchKey;
+                dialog.FindButton.Click();
+            }
+            int res = dialog.Run();
+            GroupChatModel groupChat = dialog.GroupChat;
+            dialog.Destroy();
+            if (res != (int) Gtk.ResponseType.Ok) {
+                return;
+            }
+
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    manager.OpenChat(Frontend.FrontendManager, groupChat);
+                } catch (Exception ex) {
+                    Frontend.ShowException(this, ex);
+                }
+            });
+        }
         
         protected virtual void OnChatClearAllActivityButtonClicked(object sender, EventArgs e)
         {
@@ -917,13 +1038,18 @@ namespace Smuxi.Frontend.Gnome
         
         protected virtual void OnNotebookSwitchPage(object sender, EventArgs e)
         {
-            Trace.Call(sender, e);
-            
             try {
-                _CloseChatMenuItem.Sensitive = !(_Notebook.CurrentChatView is SessionChatView);
+                var chatView = Notebook.CurrentChatView;
+                if (chatView == null) {
+                    return;
+                }
+
+                _OpenChatMenuItem.Sensitive = !(chatView is SessionChatView);
+                _CloseChatMenuItem.Sensitive = !(chatView is SessionChatView);
+                _FindGroupChatMenuItem.Sensitive = !(chatView is SessionChatView);
                 if (Frontend.IsLocalEngine) {
                     _OpenLogChatMenuItem.Sensitive =
-                        File.Exists(_Notebook.CurrentChatView.ChatModel.LogFile);
+                        File.Exists(chatView.ChatModel.LogFile);
                 }
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
@@ -1049,26 +1175,51 @@ namespace Smuxi.Frontend.Gnome
         protected void OnChatViewManagerChatAdded(object sender, ChatViewManagerChatAddedEventArgs e)
         {
             Trace.Call(sender, e);
-            
-            e.ChatView.OutputMessageTextView.MessageHighlighted += OnChatViewMessageHighlighted;
+
+            e.ChatView.MessageHighlighted += OnChatViewMessageHighlighted;
+            UpdateProgressBar();
         }
         
+        protected void OnChatViewManagerChatSynced(object sender, ChatViewManagerChatSyncedEventArgs e)
+        {
+            Trace.Call(sender, e);
+
+            UpdateProgressBar();
+        }
+
         protected void OnChatViewManagerChatRemoved(object sender, ChatViewManagerChatRemovedEventArgs e)
         {
             Trace.Call(sender, e);
             
-            e.ChatView.OutputMessageTextView.MessageHighlighted -= OnChatViewMessageHighlighted;
+            e.ChatView.MessageHighlighted -= OnChatViewMessageHighlighted;
         }
         
-        protected void OnChatViewMessageHighlighted(object sender, MessageTextViewMessageHighlightedEventArgs e)
+        protected void OnChatViewMessageHighlighted(object sender, ChatViewMessageHighlightedEventArgs e)
         {
+#if MSG_DEBUG
             Trace.Call(sender, e);
-            
+#endif
+
             if (!HasToplevelFocus) {
                 UrgencyHint = true;
             }
         }
 
+        public void UpdateProgressBar()
+        {
+            var totalChatCount = _ChatViewManager.Chats.Count;
+            var syncedChatCount =  _ChatViewManager.SyncedChats.Count;
+            _ProgressBar.Fraction = (double)syncedChatCount / totalChatCount;
+            _ProgressBar.Text = String.Format("{0} / {1}",
+                                              syncedChatCount,
+                                              totalChatCount);
+            if (syncedChatCount >= totalChatCount) {
+                _ProgressBar.Hide();
+            } else {
+                _ProgressBar.Show();
+            }
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
