@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2007-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -27,7 +21,7 @@
  */
 
 using System;
-
+using System.Threading;
 using Smuxi.Common;
 using Smuxi.Engine;
 
@@ -60,26 +54,21 @@ namespace Smuxi.Frontend.Gnome
                 return f_IsBrowseModeEnabled;
             }
             set {
-                if (value && !f_IsBrowseModeEnabled) {
+                f_IsBrowseModeEnabled = value;
+                if (value) {
 #if LOG4NET
                     f_Logger.Debug("set_IsBrowseModeEnabled(): enabling browse mode");
 #endif
-                    SwitchPage -= OnBeforeSwitchPage;
-                    SwitchPage -= OnSwitchPage;
-                }
-                if (!value && f_IsBrowseModeEnabled) {
+                } else {
 #if LOG4NET
                     f_Logger.Debug("set_IsBrowseModeEnabled(): disabling browse mode");
 #endif
-                    SwitchPage += OnBeforeSwitchPage;
-                    SwitchPage += OnSwitchPage;
-
-                    // HACK: force switch page event
+                    // HACK: force switch page event to clear activity and
+                    // update the markerline
                     var chat = CurrentChatView;
                     CurrentChatView = null;
                     CurrentChatView = chat;
                 }
-                f_IsBrowseModeEnabled = value;
             }
         }
 
@@ -94,8 +83,9 @@ namespace Smuxi.Frontend.Gnome
             Scrollable = true;
             SwitchPage += OnSwitchPage;
             SwitchPage += OnBeforeSwitchPage;
+            PageReordered += OnPageReordered;
         }
-        
+
         public void ApplyConfig(UserConfig userConfig)
         {
             switch ((string) userConfig["Interface/Notebook/TabPosition"]) {
@@ -160,7 +150,8 @@ namespace Smuxi.Frontend.Gnome
             // this also breaks the Frontend.ReconnectEngineToGUI() as that one
             // has to cleanup all chats regardless of a working network
             // connection
-            IsBrowseModeEnabled = true;
+            SwitchPage -= OnBeforeSwitchPage;
+            SwitchPage -= OnSwitchPage;
 
             int npages = NPages;
             CurrentPage = 0;
@@ -174,13 +165,20 @@ namespace Smuxi.Frontend.Gnome
             }
 
             // reconnect the event handler
-            IsBrowseModeEnabled = false;
+            SwitchPage += OnBeforeSwitchPage;
+            SwitchPage += OnSwitchPage;
         }
 
         public void SyncPagePositions()
         {
             Trace.Call();
 
+            if (Frontend.EngineVersion >= new Version("0.8.1.2")) {
+                // no need to sync chat positions with 0.8.1.2 as they get
+                // updated via Session.MoveChat()
+                return;
+            }
+
             for (int i = 0; i < NPages; i++) {
                 var chatView = (ChatView) GetNthPage(i);
                 chatView.ChatModel.Position = i;
@@ -214,13 +212,17 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
 
 #if LOG4NET
-            f_Logger.Debug("OnSwitchPageQueueAbortedEvent(): task queue aborted!");
+            f_Logger.Fatal("OnSwitchPageQueueAbortedEvent(): task queue aborted!");
 #endif
         }
 
         [GLib.ConnectBefore]
         protected virtual void OnBeforeSwitchPage(object sender, Gtk.SwitchPageArgs e)
         {
+            if (f_IsBrowseModeEnabled) {
+                return;
+            }
+
             var chatView = CurrentChatView;
             chatView.OutputMessageTextView.UpdateMarkerline();
         }
@@ -228,7 +230,11 @@ namespace Smuxi.Frontend.Gnome
         protected virtual void OnSwitchPage(object sender, Gtk.SwitchPageArgs e)
         {
             Trace.Call(sender, e);
-            
+
+            if (f_IsBrowseModeEnabled) {
+                return;
+            }
+
             // synchronize FrontManager.CurrenPage
             ChatView chatView = GetChat((int)e.PageNum);
             if (chatView == null) {
@@ -248,38 +254,67 @@ namespace Smuxi.Frontend.Gnome
                 // a non-main (GUI) thread!
                 Trace.Call(method, null, null);
 
-                DateTime start = DateTime.UtcNow, stop;
-                // OPT-TODO: we could use here a TaskStack instead which
-                // would make sure only the newest task gets executed
-                // instead of every task in the FIFO sequence!
-                // REMOTING CALL 1
-                IProtocolManager nmanager = chatModel.ProtocolManager;
+                try {
+                    DateTime start = DateTime.UtcNow, stop;
+                    // OPT-TODO: we could use here a TaskStack instead which
+                    // would make sure only the newest task gets executed
+                    // instead of every task in the FIFO sequence!
+                    // REMOTING CALL 1
+                    IProtocolManager nmanager = chatModel.ProtocolManager;
 
-                // TODO: only set the protocol manager and update network
-                // status if the protocol manager differs from the old one
+                    // TODO: only set the protocol manager and update network
+                    // status if the protocol manager differs from the old one
 
-                // REMOTING CALL 2
-                Frontend.FrontendManager.CurrentChat = chatModel;
-                if (nmanager != null) {
-                    // REMOTING CALL 3
-                    Frontend.FrontendManager.CurrentProtocolManager = nmanager;
-                }
+                    // REMOTING CALL 2
+                    Frontend.FrontendManager.CurrentChat = chatModel;
+                    if (nmanager != null) {
+                        // REMOTING CALL 3
+                        Frontend.FrontendManager.CurrentProtocolManager = nmanager;
+                    }
 
-                // even when we have no network manager, we still want to update
-                // the network status and title
-                // REMOTING CALL 4
-                Frontend.FrontendManager.UpdateNetworkStatus();
+                    // even when we have no network manager, we still want to update
+                    // the network status and title
+                    // REMOTING CALL 4
+                    Frontend.FrontendManager.UpdateNetworkStatus();
 
-                // update last seen highlight
-                // REMOTING CALL 5
-                if (lastMsg != null) {
-                    chatModel.LastSeenHighlight = lastMsg.TimeStamp;
-                }
+                    // update last seen highlight
+                    // REMOTING CALL 5
+                    if (lastMsg != null && !Frontend.UseLowBandwidthMode) {
+                        chatModel.LastSeenHighlight = lastMsg.TimeStamp;
+                    }
 
-                stop = DateTime.UtcNow;
+                    stop = DateTime.UtcNow;
 #if LOG4NET
-                f_Logger.Debug("OnSwitchPage(): task took: " + (stop - start).Milliseconds + " ms");
+                    f_Logger.Debug("OnSwitchPage(): task took: " + (stop - start).Milliseconds + " ms");
 #endif
+                } catch (Exception ex) {
+#if LOG4NET
+                    f_Logger.Error("OnSwitchPage(): Exception", ex);
+#endif
+                    Frontend.ShowException(ex);
+                }
+            });
+        }
+
+        protected virtual void OnPageReordered(object sender, Gtk.PageReorderedArgs e)
+        {
+            Trace.Call(sender, e);
+
+            if (Frontend.EngineVersion < new Version("0.8.1.2")) {
+                // Session.MoveChat() was added in >= 0.8.1.2
+                return;
+            }
+
+            var chatView = (ChatView) e.P0;
+            var newPosition = (int) e.P1;
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    Frontend.Session.MoveChat(chatView.ChatModel, newPosition);
+                } catch (Exception ex) {
+#if LOG4NET
+                    f_Logger.Error("OnPageReordered(): Exception", ex);
+#endif
+                }
             });
         }
     }
