@@ -22,6 +22,7 @@
 
 using System;
 using System.IO;
+using System.Linq;
 using Smuxi.Engine;
 
 namespace Smuxi.Frontend.Gnome
@@ -142,13 +143,29 @@ namespace Smuxi.Frontend.Gnome
             }
             Pango.FontDescription fontDescription = new Pango.FontDescription();
             if (String.IsNullOrEmpty(fontFamily)) {
-                // HACK: use fixed-sys by default if present
-                if (File.Exists("Fixedsys500c.ttf")) {
-                    fontDescription.Family = "FixedsysTTF, monospace";
-                    // fixed-sys only looks good in size 11
-                    fontDescription.Size = 11 * 1024;
-                    fontDescription.Weight = Pango.Weight.Bold;
-                    fontDescription.Style = Pango.Style.Normal;
+                // HACK: use Consolas or Fixed-Sys on Windows by default
+                if (Environment.OSVersion.Platform == PlatformID.Win32NT) {
+                    var osVersion = Environment.OSVersion.Version;
+                    var isWinXp = osVersion.Major == 5 && osVersion.Minor > 0;
+                    var context = Frontend.MainWindow.CreatePangoContext();
+                    if (context.Families.Any(family =>
+                                             family.Name == "Consolas") &&
+                        !isWinXp) {
+                        // this system has Consolas available and is not WinXP,
+                        // let's use it!
+                        fontDescription.Family = "Consolas, monospace";
+                        // Consolas only looks good in size 11
+                        fontDescription.Size = 11 * 1024;
+                        fontDescription.Weight = Pango.Weight.Normal;
+                        fontDescription.Style = Pango.Style.Normal;
+                    } else {
+                        // too bad, fallback to FixedSys then
+                        fontDescription.Family = "FixedsysTTF, monospace";
+                        // FixedSys only looks good in size 11
+                        fontDescription.Size = 11 * 1024;
+                        fontDescription.Weight = Pango.Weight.Bold;
+                        fontDescription.Style = Pango.Style.Normal;
+                    }
                 } else {
                     // use Monospace and Bold by default
                     fontDescription.Family = "monospace";
