@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2010 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -28,6 +22,7 @@
 
 using System;
 using System.Drawing;
+using System.Threading;
 using System.Collections.Generic;
 using Smuxi.Common;
 using Smuxi.Engine;
@@ -36,15 +31,16 @@ using Smuxi.Frontend;
 namespace Smuxi.Frontend.Gnome
 {
     // TODO: use Gtk.Bin
-    public abstract class ChatView : Gtk.EventBox, IChatView, IDisposable
+    public abstract class ChatView : Gtk.EventBox, IChatView, IDisposable, ITraceable
     {
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
-        public    string             ID { get; private set; }
-        private   string             _Name;
+        public    string             ID { get; internal set; }
+        public    int                Position { get; internal set; }
         private   ChatModel          _ChatModel;
         private   bool               _HasHighlight;
+        private   int                HighlightCount { get; set; }
         private   bool               _HasActivity;
         private   bool               _HasEvent;
         private   bool               _IsSynced;
@@ -56,15 +52,31 @@ namespace Smuxi.Frontend.Gnome
         private   Gtk.ScrolledWindow _OutputScrolledWindow;
         private   MessageTextView    _OutputMessageTextView;
         private   ThemeSettings      _ThemeSettings;
-        private   DateTime           _LastHighlight;
         private   TaskQueue          _LastSeenHighlightQueue;
-        
+        public    DateTime           SyncedLastSeenHighlight { get; private set; }
+        IList<MessageModel>          SyncedMessages { get; set; }
+        protected string             SyncedName { get; set; }
+        public    IProtocolManager   ProtocolManager { get; set; }
+        bool                         UseLowBandwidthMode { get; set; }
+        protected Gtk.Image          TabImage { get; set; }
+        bool                         IsAutoScrolling { get; set; }
+
         public ChatModel ChatModel {
             get {
                 return _ChatModel;
             }
         }
 
+        public new string Name {
+            get {
+                return base.Name;
+            }
+            set {
+                base.Name = value;
+                _TabLabel.Text = value;
+            }
+        }
+
         // this property is thread-safe
         public bool IsActive {
             get {
@@ -84,24 +96,37 @@ namespace Smuxi.Frontend.Gnome
             }
             set {
                 _HasHighlight = value;
-                
+                HighlightCount++;
+
                 if (!value) {
                     // clear highlight with "no activity"
                     HasActivity = false;
+                    HighlightCount = 0;
                     return;
                 }
 
-                var color = ColorTools.GetBestTextColor(
-                    ColorTools.GetTextColor(_ThemeSettings.HighlightColor),
-                    ColorTools.GetTextColor(
+                var color = TextColorTools.GetBestTextColor(
+                    ColorConverter.GetTextColor(_ThemeSettings.HighlightColor),
+                    ColorConverter.GetTextColor(
                         Gtk.Rc.GetStyle(_TabLabel).Base(Gtk.StateType.Normal)
-                    ), ColorContrast.High
-                );
-                _TabLabel.Markup = String.Format(
-                    "<span foreground=\"{0}\">{1}</span>",
-                    GLib.Markup.EscapeText(color.ToString()),
-                    GLib.Markup.EscapeText(_Name)
+                    ), TextColorContrast.High
                 );
+
+                if (HighlightCount > 1) {
+                    _TabLabel.Markup = String.Format(
+                        "<span foreground=\"{0}\">{1} ({2})</span>",
+                        GLib.Markup.EscapeText(color.ToString()),
+                        GLib.Markup.EscapeText(Name),
+                        GLib.Markup.EscapeText(HighlightCount.ToString())
+
+                    );
+                } else {
+                    _TabLabel.Markup = String.Format(
+                        "<span foreground=\"{0}\">{1}</span>",
+                        GLib.Markup.EscapeText(color.ToString()),
+                        GLib.Markup.EscapeText(Name)
+                    );
+                }
             }
         }
 
@@ -110,6 +135,10 @@ namespace Smuxi.Frontend.Gnome
                 return _HasActivity;
             }
             set {
+                if (value && _HasActivity == value) {
+                    // nothing to update
+                    return;
+                }
                 _HasActivity = value;
 
                 if (HasHighlight) {
@@ -123,16 +152,16 @@ namespace Smuxi.Frontend.Gnome
                 } else {
                     colorValue = _ThemeSettings.NoActivityColor;
                 }
-                var color = ColorTools.GetBestTextColor(
-                    ColorTools.GetTextColor(colorValue),
-                    ColorTools.GetTextColor(
+                var color = TextColorTools.GetBestTextColor(
+                    ColorConverter.GetTextColor(colorValue),
+                    ColorConverter.GetTextColor(
                         Gtk.Rc.GetStyle(_TabLabel).Base(Gtk.StateType.Normal)
-                    ), ColorContrast.High
+                    ), TextColorContrast.High
                 );
                 _TabLabel.Markup = String.Format(
                     "<span foreground=\"{0}\">{1}</span>",
                     GLib.Markup.EscapeText(color.ToString()),
-                    GLib.Markup.EscapeText(_Name)
+                    GLib.Markup.EscapeText(Name)
                 );
             }
         }
@@ -142,6 +171,8 @@ namespace Smuxi.Frontend.Gnome
                 return _HasEvent;
             }
             set {
+                _HasEvent = value;
+
                 if (HasHighlight) {
                     return;
                 }
@@ -155,16 +186,16 @@ namespace Smuxi.Frontend.Gnome
                     return;
                 }
 
-                var color = ColorTools.GetBestTextColor(
-                    ColorTools.GetTextColor(_ThemeSettings.EventColor),
-                    ColorTools.GetTextColor(
+                var color = TextColorTools.GetBestTextColor(
+                    ColorConverter.GetTextColor(_ThemeSettings.EventColor),
+                    ColorConverter.GetTextColor(
                         Gtk.Rc.GetStyle(_TabLabel).Base(Gtk.StateType.Normal)
-                    ), ColorContrast.High
+                    ), TextColorContrast.High
                 );
                 _TabLabel.Markup = String.Format(
                     "<span foreground=\"{0}\">{1}</span>",
                     GLib.Markup.EscapeText(color.ToString()),
-                    GLib.Markup.EscapeText(_Name)
+                    GLib.Markup.EscapeText(Name)
                 );
             }
         }
@@ -214,15 +245,19 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        protected abstract Gtk.Image DefaultTabImage {
+            get;
+        }
+
+        public event EventHandler<ChatViewMessageHighlightedEventArgs> MessageHighlighted;
+
         public ChatView(ChatModel chat)
         {
             Trace.Call(chat);
             
             _ChatModel = chat;
-            _Name = _ChatModel.Name;
-            ID = _ChatModel.ID;
-            Name = _Name;
-            
+
+            IsAutoScrolling = true;
             MessageTextView tv = new MessageTextView();
             _EndMark = tv.Buffer.CreateMark("end", tv.Buffer.EndIter, false); 
             tv.ShowTimestamps = true;
@@ -233,32 +268,40 @@ namespace Smuxi.Frontend.Gnome
             tv.MessageAdded += OnMessageTextViewMessageAdded;
             tv.MessageHighlighted += OnMessageTextViewMessageHighlighted;
             tv.PopulatePopup += OnMessageTextViewPopulatePopup;
+            tv.SizeRequested += delegate {
+                AutoScroll();
+            };
             _OutputMessageTextView = tv;
 
             Gtk.ScrolledWindow sw = new Gtk.ScrolledWindow();
+            _OutputScrolledWindow = sw;
             //sw.HscrollbarPolicy = Gtk.PolicyType.Never;
             sw.HscrollbarPolicy = Gtk.PolicyType.Automatic;
             sw.VscrollbarPolicy = Gtk.PolicyType.Always;
             sw.ShadowType = Gtk.ShadowType.In;
+            sw.Vadjustment.ValueChanged += delegate {
+                CheckAutoScroll();
+            };
             sw.Add(_OutputMessageTextView);
-            _OutputScrolledWindow = sw;
-            
+
             // popup menu
             _TabMenu = new Gtk.Menu();
             
             Gtk.ImageMenuItem close_item = new Gtk.ImageMenuItem(Gtk.Stock.Close, null);
             close_item.Activated += new EventHandler(OnTabMenuCloseActivated);  
             _TabMenu.Append(close_item);
+            _TabMenu.ShowAll();
             
             //FocusChild = _OutputTextView;
             //CanFocus = false;
             
             _TabLabel = new Gtk.Label();
-            _TabLabel.Text = _Name;
-            
+
+            TabImage = DefaultTabImage;
             _TabHBox = new Gtk.HBox();
             _TabHBox.PackEnd(new Gtk.Fixed(), true, true, 0);
             _TabHBox.PackEnd(_TabLabel, false, false, 0);
+            _TabHBox.PackStart(TabImage, false, false, 2);
             _TabHBox.ShowAll();
             
             _TabEventBox = new Gtk.EventBox();
@@ -270,7 +313,7 @@ namespace Smuxi.Frontend.Gnome
             _ThemeSettings = new ThemeSettings();
 
             // OPT-TODO: this should use a TaskStack instead of TaskQueue
-            _LastSeenHighlightQueue = new TaskQueue("LastSeenHighlightQueue("+_Name+")");
+            _LastSeenHighlightQueue = new TaskQueue("LastSeenHighlightQueue("+ID+")");
             _LastSeenHighlightQueue.AbortedEvent += OnLastSeenHighlightQueueAbortedEvent;
             _LastSeenHighlightQueue.ExceptionEvent += OnLastSeenHighlightQueueExceptionEvent;
         }
@@ -337,17 +380,18 @@ namespace Smuxi.Frontend.Gnome
         
         public virtual void ScrollToEnd()
         {
+#if SCROLL_DEBUG
             Trace.Call();
-            
-            Gtk.Adjustment adj = _OutputScrolledWindow.Vadjustment;
-#if LOG4NET
+#endif
+
+            // BUG? doesn't work always for some reason
+            // seems like GTK+ doesn't update the adjustment till we give back control
+            //Gtk.Adjustment adj = _OutputScrolledWindow.Vadjustment;
+#if LOG4NET && SCROLL_DEBUG
             _Logger.Debug("ScrollToEnd(): Vadjustment.Value: " + adj.Value +
                           " Vadjustment.Upper: " + adj.Upper +
                           " Vadjustment.PageSize: " + adj.PageSize);
 #endif
-            
-            // BUG? doesn't work always for some reason
-            // seems like GTK+ doesn't update the adjustment till we give back control
             //adj.Value = adj.Upper - adj.PageSize;
             
             //_OutputTextView.Buffer.MoveMark(_EndMark, _OutputTextView.Buffer.EndIter);
@@ -357,8 +401,10 @@ namespace Smuxi.Frontend.Gnome
             //_OutputTextView.ScrollMarkOnscreen(_OutputTextView.Buffer.InsertMark);
 
             //_OutputTextView.ScrollMarkOnscreen(_OutputTextView.Buffer.GetMark("tail"));
-            
+
+#if SCROLL_DEBUG
             System.Reflection.MethodBase mb = Trace.GetMethodBase();
+#endif
             // WORKAROUND1: scroll after one second delay
             /*
             GLib.Timeout.Add(1000, new GLib.TimeoutHandler(delegate {
@@ -370,13 +416,34 @@ namespace Smuxi.Frontend.Gnome
             */
             // WORKAROUND2: scroll when GTK+ mainloop is idle
             GLib.Idle.Add(new GLib.IdleHandler(delegate {
+#if SCROLL_DEBUG
                 Trace.Call(mb);
-                
+#endif
                 _OutputMessageTextView.ScrollMarkOnscreen(_EndMark);
                 return false;
             }));
         }
         
+        void CheckAutoScroll()
+        {
+            var vAdjustment = _OutputScrolledWindow.Vadjustment;
+            if (vAdjustment.Upper == (vAdjustment.Value + vAdjustment.PageSize)) {
+                // the scrollbar is way at the end, lets autoscroll
+                IsAutoScrolling = true;
+            } else {
+                IsAutoScrolling = false;
+            }
+        }
+
+        void AutoScroll()
+        {
+            if (!IsAutoScrolling) {
+                return;
+            }
+
+            ScrollToEnd();
+        }
+
         public virtual void Enable()
         {
             Trace.Call();
@@ -392,31 +459,83 @@ namespace Smuxi.Frontend.Gnome
         public virtual void Sync()
         {
             Trace.Call();
-            
+
+            GLib.Idle.Add(delegate {
+                TabImage.SetFromStock(Gtk.Stock.Refresh, Gtk.IconSize.Menu);
+                return false;
+            });
+
+            // REMOTING CALL
+            SyncedName = _ChatModel.Name;
+            // REMOTING CALL
+            ProtocolManager = _ChatModel.ProtocolManager;
+
+            if (!Frontend.IsLocalEngine && Frontend.UseLowBandwidthMode) {
+                // FIXME: set TabImage back to normal
+                return;
+            }
+
+            // REMOTING CALL
+            SyncedLastSeenHighlight = _ChatModel.LastSeenHighlight;
+
+            DateTime start, stop;
+            start = DateTime.UtcNow;
+            // REMOTING CALL
+            SyncedMessages = _ChatModel.Messages;
+            stop = DateTime.UtcNow;
 #if LOG4NET
-            _Logger.Debug("Sync() syncing messages");
+            _Logger.Debug(
+                String.Format(
+                    "Sync(): retrieving ChatModel.Messages took: {0:0.00} ms",
+                    (stop - start).TotalMilliseconds
+                )
+            );
 #endif
+        }
+
+        public virtual void Populate()
+        {
+            Trace.Call();
+
+            Name = SyncedName;
+
             // sync messages
             // cleanup, be sure the output is empty
             _OutputMessageTextView.Clear();
-            IList<MessageModel> messages = _ChatModel.Messages;
-            if (messages.Count > 0) {
-                foreach (MessageModel msg in messages) {
-                    AddMessage(msg);
+
+            if (!Frontend.IsLocalEngine && Frontend.UseLowBandwidthMode) {
+                var msg = new MessageBuilder();
+                msg.AppendEventPrefix();
+                msg.AppendMessage(_("Low Bandwidth Mode is active: no messages synced."));
+                AddMessage(msg.ToMessage());
+            } else {
+                if (SyncedMessages.Count > 0) {
+                    // TODO: push messages in batches and give back control to
+                    // GTK+ in between for blocking the GUI thread less
+                    foreach (MessageModel msg in SyncedMessages) {
+                        AddMessage(msg);
+                    }
                 }
             }
-            // REMOTING CALL
-            if (_LastHighlight > _ChatModel.LastSeenHighlight) {
-                HasHighlight = true;
+
+            // as we don't track which messages were already seen it would
+            // show all chats with message activity after the frontend connect
+            if (!HasHighlight) {
+                HasActivity = false;
             }
 
+            // let the user know at which position new messages start
+            _OutputMessageTextView.UpdateMarkerline();
+
+            // reset tab icon to normal
+            TabImage.Pixbuf = DefaultTabImage.Pixbuf;
+
+            SyncedMessages = null;
             _IsSynced = true;
         }
         
         public virtual void AddMessage(MessageModel msg)
         {
-            Trace.Call(msg);
-            
             _OutputMessageTextView.AddMessage(msg);
         }
         
@@ -444,35 +563,54 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
 
-            ChatModel.ProtocolManager.CloseChat(
-                Frontend.FrontendManager,
-                ChatModel
-            );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    ChatModel.ProtocolManager.CloseChat(
+                        Frontend.FrontendManager,
+                        ChatModel
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
+        }
+
+        public override string ToString()
+        {
+            return String.Format("<{0}>", ToTraceString());
+        }
+
+        public string ToTraceString()
+        {
+            return ID;
         }
 
         protected virtual void OnTabButtonPress(object sender, Gtk.ButtonPressEventArgs e)
         {
             Trace.Call(sender, e);
-            
-            if (e.Event.Button == 3) {
-                _TabMenu.Popup(null, null, null, e.Event.Button, e.Event.Time);
-                _TabMenu.ShowAll();
-            } else if (e.Event.Button == 2) {
-                Close();
+
+            try {
+                if (e.Event.Button == 3) {
+                    _TabMenu.Popup(null, null, null, e.Event.Button, e.Event.Time);
+                }
+            } catch (Exception ex) {
+                Frontend.ShowException(ex);
             }
         }
         
         protected virtual void OnTabMenuCloseActivated(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
-            
-            Close();
+
+            try {
+                Close();
+            } catch (Exception ex) {
+                Frontend.ShowException(ex);
+            }
         }
         
         protected virtual void OnMessageTextViewMessageAdded(object sender, MessageTextViewMessageAddedEventArgs e)
         {
-            Trace.Call(sender, e);
-            
             // HACK: out of scope?
             // probably we should use the ChatViewManager instead?
             if (Frontend.MainWindow.Notebook.CurrentChatView != this) {
@@ -486,50 +624,22 @@ namespace Smuxi.Frontend.Gnome
                 }
             }
 
-            Gtk.ScrolledWindow sw = _OutputScrolledWindow;
-            Gtk.TextView tv = _OutputMessageTextView;
+            var buffer = _OutputMessageTextView.Buffer;
+            buffer.MoveMark(_EndMark, buffer.EndIter);
 
-            if (sw.Vadjustment.Upper == (sw.Vadjustment.Value + sw.Vadjustment.PageSize)) {
-                // the scrollbar is way at the end, lets autoscroll
-                Gtk.TextIter endit = tv.Buffer.EndIter;
-                tv.Buffer.PlaceCursor(endit);
-                tv.Buffer.MoveMark(tv.Buffer.InsertMark, endit);
-                tv.ScrollMarkOnscreen(tv.Buffer.InsertMark);
-            }
-
-            // update the end mark
-            tv.Buffer.MoveMark(_EndMark, tv.Buffer.EndIter);
+            AutoScroll();
         }
         
         protected virtual void OnMessageTextViewMessageHighlighted(object sender, MessageTextViewMessageHighlightedEventArgs e)
         {
-            Trace.Call(sender, e);
-            
-            // HACK: out of scope?
-            // only beep if the main windows has no focus (the user is
-            // elsewhere) and the chat is was already synced, as during sync we
-            // would get insane from all beeping caused by the old highlights
-            if (!Frontend.MainWindow.HasToplevelFocus &&
-                _IsSynced &&
-                Frontend.UserConfig["Sound/BeepOnHighlight"] != null &&
-                (bool) Frontend.UserConfig["Sound/BeepOnHighlight"]) {
-#if LOG4NET
-                _Logger.Debug("OnMessageTextViewMessageHighlighted(): BEEP!");
-#endif
-                try {
-                    if (Display != null) {
-                        Display.Beep();
-                    }
-                } catch (Exception ex) {
-#if LOG4NET
-                    _Logger.Error("OnMessageTextViewMessageHighlighted(): Exception", ex);
-#endif
-                }
-            }
-
             if (_IsSynced) {
                 bool isActiveChat = IsActive;
 
+                if (Frontend.UseLowBandwidthMode && !isActiveChat) {
+                    HasHighlight = true;
+                    return;
+                }
+
                 var method = Trace.GetMethodBase();
                 // update last seen highlight
                 // OPT-TODO: we should use a TaskStack here OR at least a
@@ -565,7 +675,39 @@ namespace Smuxi.Frontend.Gnome
                     }
                 });
             } else {
-                _LastHighlight = e.Message.TimeStamp;
+                if (e.Message.TimeStamp > SyncedLastSeenHighlight) {
+                    HasHighlight = true;
+                }
+            }
+
+            if (e.Message.TimeStamp > SyncedLastSeenHighlight) {
+                // unseen highlight
+
+                // HACK: out of scope?
+                // only beep if the main windows has no focus (the user is
+                // elsewhere) and the chat is was already synced, as during sync we
+                // would get insane from all beeping caused by the old highlights
+                if (!Frontend.MainWindow.HasToplevelFocus &&
+                    _IsSynced &&
+                    Frontend.UserConfig["Sound/BeepOnHighlight"] != null &&
+                    (bool) Frontend.UserConfig["Sound/BeepOnHighlight"]) {
+#if LOG4NET
+                    _Logger.Debug("OnMessageTextViewMessageHighlighted(): BEEP!");
+#endif
+                    try {
+                        if (Display != null) {
+                            Display.Beep();
+                        }
+                    } catch (Exception ex) {
+#if LOG4NET
+                        _Logger.Error("OnMessageTextViewMessageHighlighted(): Exception", ex);
+#endif
+                    }
+                }
+
+                if (MessageHighlighted != null) {
+                    MessageHighlighted(this, new ChatViewMessageHighlightedEventArgs(e.Message));
+                }
             }
         }
 
@@ -576,9 +718,6 @@ namespace Smuxi.Frontend.Gnome
             if (OutputMessageTextView.IsAtUrlTag) {
                 return;
             }
-            if (Frontend.MainWindow.ShowMenuBar) {
-                return;
-            }
 
             Gtk.Menu popup = e.Menu;
             popup.Prepend(new Gtk.SeparatorMenuItem());
@@ -587,7 +726,7 @@ namespace Smuxi.Frontend.Gnome
             item.Active = Frontend.MainWindow.ShowMenuBar;
             item.Activated += delegate {
                 try {
-                    Frontend.MainWindow.ShowMenuBar = true;
+                    Frontend.MainWindow.ShowMenuBar = !Frontend.MainWindow.ShowMenuBar;
                 } catch (Exception ex) {
                     Frontend.ShowException(ex);
                 }
@@ -621,4 +760,14 @@ namespace Smuxi.Frontend.Gnome
             return Mono.Unix.Catalog.GetString(msg);
         }
     }
+
+    public class ChatViewMessageHighlightedEventArgs : EventArgs
+    {
+        public MessageModel Message { get; set; }
+
+        public ChatViewMessageHighlightedEventArgs(MessageModel msg)
+        {
+            Message = msg;
+        }
+    }
 }
