@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2006, 2009-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -27,6 +21,7 @@
  */
 
 using System;
+using System.Threading;
 using System.Collections.Generic;
 using System.Globalization;
 using Mono.Unix;
@@ -55,6 +50,8 @@ namespace Smuxi.Frontend.Gnome
         private MessageTextView    _TopicTextView;
         private MessageModel       _Topic;
         private Gtk.TreeViewColumn _IdentityNameColumn;
+        IDictionary<string, PersonModel> SyncedPersons { get; set; }
+        MessageModel                     SyncedTopic  { get; set; }
 
         public override bool HasSelection {
             get {
@@ -103,10 +100,19 @@ namespace Smuxi.Frontend.Gnome
                 return _IdentityNameColumn;
             }
         }
-        
-        static GroupChatView() {
-            IconPixbuf = new Gdk.Pixbuf(null, "group-chat.svg", 16, 16);
-       }
+
+        protected override Gtk.Image DefaultTabImage {
+            get {
+                return new Gtk.Image(IconPixbuf);
+            }
+        }
+
+        static GroupChatView()
+        {
+            IconPixbuf = Frontend.LoadIcon(
+                "smuxi-group-chat", 16, "group-chat_256x256.png"
+            );
+        }
 
         public GroupChatView(GroupChatModel groupChat) : base(groupChat)
         {
@@ -183,22 +189,9 @@ namespace Smuxi.Frontend.Gnome
             _TopicScrolledWindow.Visible = false;
             _TopicScrolledWindow.NoShowAll = true;
 
-            // predict and set useful topic heigth
-            Pango.Layout layout = _TopicTextView.CreatePangoLayout("Test Topic");
-            int lineWidth, lineHeigth;
-            layout.GetPixelSize(out lineWidth, out lineHeigth);
-            // use 2 lines + a bit extra as the topic heigth
-            int bestHeigth = (lineHeigth * 2) + 5;
-            _TopicTextView.HeightRequest = bestHeigth;
-            _TopicScrolledWindow.HeightRequest = bestHeigth;
-            
             Add(_OutputHPaned);
             
-            ApplyConfig(Frontend.UserConfig);
-            
-            var tabImage = new Gtk.Image(IconPixbuf);
-            TabHBox.PackStart(tabImage, false, false, 2);
-            TabHBox.ShowAll();
+            //ApplyConfig(Frontend.UserConfig);
             
             ShowAll();
         }
@@ -218,54 +211,53 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
 
-            IDictionary<string, PersonModel> persons = _GroupChatModel.Persons;
-            if (persons == null) {
-                persons = new Dictionary<string, PersonModel>(0);
-            }
+            GLib.Idle.Add(delegate {
+                TabImage.SetFromStock(Gtk.Stock.Refresh, Gtk.IconSize.Menu);
+                return false;
+            });
+
 #if LOG4NET
             _Logger.Debug("Sync() syncing persons");
 #endif
+            // REMOTING CALL 1
+            SyncedPersons = _GroupChatModel.Persons;
+            if (SyncedPersons == null) {
+                SyncedPersons = new Dictionary<string, PersonModel>(0);
+            }
+
+#if LOG4NET
+            _Logger.Debug("Sync() syncing topic");
+#endif
+            // REMOTING CALL 2
+            SyncedTopic = _GroupChatModel.Topic;
+
+            base.Sync();
+        }
+
+        public override void Populate()
+        {
+            Trace.Call();
+
             // sync persons
             if (_PersonTreeView != null) {
-                int count = persons.Count;
-                /*
-                if (count > 1) {
-                    Frontend.MainWindow.ProgressBar.DiscreteBlocks = (uint)count;
-                } else {
-                    Frontend.MainWindow.ProgressBar.DiscreteBlocks = 2;
-                }
-                Frontend.MainWindow.ProgressBar.BarStyle = Gtk.ProgressBarStyle.Continuous;
-                */
-                
                 // HACK: out of scope
                 string status = String.Format(
                                     _("Retrieving user list for {0}..."),
-                                    ChatModel.Name);
-                Frontend.MainWindow.Statusbar.Push(0, status);
+                                    SyncedName);
+                Frontend.MainWindow.Status = status;
     
                 Gtk.ListStore ls = (Gtk.ListStore) _PersonTreeView.Model;
                 // cleanup, be sure the list is empty
                 ls.Clear();
                 // detach the model (less CPU load)
                 _PersonTreeView.Model = new Gtk.ListStore(typeof(PersonModel));
-                int i = 1;
                 string longestName = String.Empty;
-                foreach (PersonModel person in persons.Values) {
+                foreach (PersonModel person in SyncedPersons.Values) {
                     ls.AppendValues(person);
                     
                     if (person.IdentityName.Length > longestName.Length) {
                         longestName = person.IdentityName;
                     }
-                    
-                    //Frontend.MainWindow.ProgressBar.Fraction = (double)i++ / count;
-                    /*
-                    // this seems to break the sync when it's remote engine is used,
-                    // guess it does some other GUI processing, like removing users from
-                    // the userlist....
-                    while (Gtk.Application.EventsPending()) {
-                        Gtk.Application.RunIteration(false);
-                    }
-                    */
                 }
                 // attach the model again
                 // BUG? TreeView doesn't seem to recognize existing values in the model?!?
@@ -283,20 +275,14 @@ namespace Smuxi.Frontend.Gnome
                 
                 UpdatePersonCount(); 
                
-                // HACK: out of scope
-                Frontend.MainWindow.ProgressBar.Fraction = 0;
                 // TRANSLATOR: this string will be appended to the one above
                 status += String.Format(" {0}", _("done."));
-                Frontend.MainWindow.Statusbar.Push(0, status);
+                Frontend.MainWindow.Status = status;
             }
 
-#if LOG4NET
-            _Logger.Debug("Sync() syncing topic");
-#endif
-            // sync topic
-            Topic = _GroupChatModel.Topic;
+            Topic = SyncedTopic;
 
-            base.Sync();
+            base.Populate();
         }
         
         protected void UpdatePersonCount()
@@ -374,6 +360,90 @@ namespace Smuxi.Frontend.Gnome
             } while (_PersonListStore.IterNext(ref iter));
             UpdatePersonCount();
         }
+
+        /*Logic taken from the PersonLookup method of Engine/Chats/GroupChatModel.cs */
+        public PersonModel PersonLookup(string identityName)
+        {
+            Trace.Call(identityName);
+#if LOG4NET
+            _Logger.Debug("PersonLookup(): GroupChatModel.Name: " + Name);
+#endif
+            int identityNameLength = identityName.Length;
+            if (_PersonListStore == null) {
+                return null;
+            }
+            foreach (object[] row in _PersonListStore) {
+                PersonModel person = (PersonModel)row[0];
+                if ((person.IdentityName.Length >= identityNameLength) &&
+                    (person.IdentityName.Substring(0, identityNameLength).ToLower() == identityName.ToLower())) {
+#if LOG4NET
+                    _Logger.Debug("PersonLookup(): found: " + person.IdentityName);
+#endif
+                    return person;
+                }
+            }
+
+#if LOG4NET
+            _Logger.Debug("PersonLookup() no matching identityName found");
+#endif
+            return null;
+        }
+
+        /*Logic taken from the PersonLookupAll method of Engine/Chats/GroupChatModel.cs */
+        public IList<string> PersonLookupAll(string identityName)
+        {
+            Trace.Call(identityName);
+
+            IList<string> foundIdentityNames = new List<string>();
+            int identityNameLength = identityName.Length;
+            string longestIdentityName = String.Empty;
+            if (_PersonListStore == null) {
+                return foundIdentityNames;
+            }
+            foreach (object[] row in _PersonListStore) {
+                PersonModel person = (PersonModel)row[0];
+                if ((person.IdentityName.Length >= identityNameLength) &&
+                    (person.IdentityName.Substring(0, identityNameLength).ToLower() == identityName.ToLower())) {
+                    foundIdentityNames.Add(person.IdentityName);
+                    if (person.IdentityName.Length > longestIdentityName.Length) {
+                        longestIdentityName = person.IdentityName;
+                    }
+                }
+            }
+
+            // guess the common part of the found nicknames
+            string common_nick = identityName;
+            bool match = true;
+            while (match) {
+                if (common_nick.Length >= longestIdentityName.Length) {
+                    break;
+                }
+                common_nick += longestIdentityName[common_nick.Length];
+                foreach (string name in foundIdentityNames) {
+                    if (!name.ToLower().StartsWith(common_nick.ToLower())) {
+                        common_nick = common_nick.Substring(0, common_nick.Length - 1);
+                        match = false;
+                     }
+                }
+            }
+
+            if (foundIdentityNames.Count == 0) {
+#if LOG4NET
+                _Logger.Debug("PersonLookupAll(): no matching identityName found");
+#endif
+            } else if (foundIdentityNames.Count == 1) {
+#if LOG4NET
+                _Logger.Debug("PersonLookupAll(): found exact match: " + foundIdentityNames[0]);
+#endif
+            } else {
+#if LOG4NET
+                _Logger.Debug("PersonLookupAll(): found " + foundIdentityNames.Count + " matches");
+#endif
+                foundIdentityNames.Insert(0, common_nick);
+            }
+            return foundIdentityNames;
+        }
+
         
         public override void ApplyConfig(UserConfig config)
         {
@@ -387,6 +457,15 @@ namespace Smuxi.Frontend.Gnome
 
             // topic
             _TopicTextView.ApplyConfig(config);
+            // predict and set useful topic heigth
+            Pango.Layout layout = _TopicTextView.CreatePangoLayout("Test Topic");
+            int lineWidth, lineHeigth;
+            layout.GetPixelSize(out lineWidth, out lineHeigth);
+            // use 2 lines + a bit extra as the topic heigth
+            int bestHeigth = (lineHeigth * 2) + 5;
+            _TopicTextView.HeightRequest = bestHeigth;
+            _TopicScrolledWindow.HeightRequest = bestHeigth;
+
             string topic_pos = (string) config["Interface/Notebook/Channel/TopicPosition"];
             if (_TopicScrolledWindow.IsAncestor(_OutputVBox)) {
                 _OutputVBox.Remove(_TopicScrolledWindow);
@@ -474,20 +553,31 @@ namespace Smuxi.Frontend.Gnome
             
             PersonModel person1 = (PersonModel) liststore.GetValue(iter1, 0); 
             PersonModel person2 = (PersonModel) liststore.GetValue(iter2, 0); 
-            
-            return String.Compare(person1.IdentityName, person2.IdentityName,
-                                  true, CultureInfo.InvariantCulture);
+
+            return person1.CompareTo(person2);
         }
-        
+
         protected virtual void OnPersonsRowActivated(object sender, Gtk.RowActivatedArgs e)
         {
             Trace.Call(sender, e);
 
             IList<PersonModel> persons = GetSelectedPersons();
-            if (persons == null) {
+            if (persons == null || persons.Count == 0) {
                 return;
             }
 
+            // jump to person chat if available
+            foreach (var chatView in Frontend.MainWindow.ChatViewManager.Chats) {
+                if (!(chatView is PersonChatView)) {
+                    continue;
+                }
+                var personChatView = (PersonChatView) chatView;
+                if (personChatView.PersonModel == persons[0]) {
+                    Frontend.MainWindow.ChatViewManager.CurrentChatView = personChatView;
+                    return;
+                }
+            }
+
             // this is a generic implemention that should be able to open/create
             // a private chat in most cases, as it depends what OpenChat()
             // of the specific protocol actually expects/needs
@@ -496,12 +586,19 @@ namespace Smuxi.Frontend.Gnome
                     person,
                     person.ID,
                     person.IdentityName,
-                    ChatModel.ProtocolManager
-                );
-                ChatModel.ProtocolManager.OpenChat(
-                    Frontend.FrontendManager,
-                    personChat
+                    null
                 );
+
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        ChatModel.ProtocolManager.OpenChat(
+                            Frontend.FrontendManager,
+                            personChat
+                        );
+                    } catch (Exception ex) {
+                        Frontend.ShowException(ex);
+                    }
+                });
             }
         }
 
@@ -527,7 +624,6 @@ namespace Smuxi.Frontend.Gnome
             if (e.Event.Key == Gdk.Key.Menu &&
                 _PersonTreeView.Selection.CountSelectedRows() > 0) {
                 _PersonMenu.Popup(null, null, null, 0, e.Event.Time);
-                _PersonMenu.ShowAll();
             }
         }
 
@@ -546,7 +642,6 @@ namespace Smuxi.Frontend.Gnome
                 // 0 == left mouse button here
                 //_PersonMenu.Popup(null, null, null, e.Event.Button, e.Event.Time);
                 _PersonMenu.Popup(null, null, null, 0, e.Event.Time);
-                _PersonMenu.ShowAll();
             }
         }
         
