@@ -1,9 +1,7 @@
 /*
- * $Id$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2009 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2009-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -45,8 +43,8 @@ namespace Smuxi.Frontend.Gnome
         private bool         _ShowTimestamps;
         private bool         _ShowHighlight;
         private bool         _ShowMarkerline;
-        private bool         _AtUrlTag;
-        private string       _Url;
+        private bool         _AtLinkTag;
+        private Uri          _ActiveLink;
         private UserConfig   _Config;
         private ThemeSettings _ThemeSettings;
         private Gdk.Color    _MarkerlineColor = new Gdk.Color(255, 0, 0);
@@ -109,7 +107,7 @@ namespace Smuxi.Frontend.Gnome
 
         public bool IsAtUrlTag {
             get {
-                return _AtUrlTag;
+                return _AtLinkTag;
             }
         }
 
@@ -176,36 +174,52 @@ namespace Smuxi.Frontend.Gnome
         
         public void AddMessage(MessageModel msg, bool addLinebreak)
         {
+#if MSG_DEBUG
             Trace.Call(msg, addLinebreak);
+#endif
 
             if (msg == null) {
                 throw new ArgumentNullException("msg");
             }
 
-            Gtk.TextIter iter = Buffer.EndIter;
+            var buffer = Buffer;
+            var iter = buffer.EndIter;
 
             Gdk.Color bgColor = DefaultAttributes.Appearance.BgColor;
             if (_ThemeSettings.BackgroundColor != null) {
                 bgColor = _ThemeSettings.BackgroundColor.Value;
             }
-            TextColor bgTextColor = ColorTools.GetTextColor(bgColor);
+            TextColor bgTextColor = ColorConverter.GetTextColor(bgColor);
 
             if (_ShowTimestamps) {
-                DateTime localTimestamp = msg.TimeStamp.ToLocalTime();
-                if (_LastMessage != null &&
-                    _LastMessage.TimeStamp.ToLocalTime().Date != localTimestamp.Date) {
-                    string dayLine = String.Format(
-                        "-!- " + _("Day changed to {0}"),
-                        localTimestamp.Date.ToLongDateString()
-                    );
-                    Buffer.Insert(ref iter, dayLine + "\n");
+                var msgTimeStamp = msg.TimeStamp.ToLocalTime();
+                if (_LastMessage != null) {
+                    var lastMsgTimeStamp = _LastMessage.TimeStamp.ToLocalTime();
+                    var span = msgTimeStamp.Date - lastMsgTimeStamp.Date;
+                    string dayLine = null;
+                    if (span.Days > 1) {
+                        dayLine = String.Format(
+                            "-!- " + _("Day changed from {0} to {1}"),
+                            lastMsgTimeStamp.ToShortDateString(),
+                            msgTimeStamp.ToShortDateString()
+                        );
+                    } else if (span.Days > 0) {
+                        dayLine = String.Format(
+                            "-!- " + _("Day changed to {0}"),
+                            msgTimeStamp.ToLongDateString()
+                        );
+                    }
+
+                    if (dayLine != null) {
+                        buffer.Insert(ref iter, dayLine + "\n");
+                    }
                 }
                 
                 string timestamp = null;
                 try {
                     string format = (string)Frontend.UserConfig["Interface/Notebook/TimestampFormat"];
                     if (!String.IsNullOrEmpty(format)) {
-                        timestamp = localTimestamp.ToString(format);
+                        timestamp = msgTimeStamp.ToString(format);
                     }
                 } catch (FormatException e) {
                     timestamp = "Timestamp Format ERROR: " + e.Message;
@@ -217,17 +231,17 @@ namespace Smuxi.Frontend.Gnome
                         // get best contrast for the event font color
                         Gtk.TextTag eventTag = _MessageTextTagTable.Lookup("event");
                         Gdk.Color eventColor = eventTag.ForegroundGdk;
-                        TextColor eventTextColor = ColorTools.GetBestTextColor(
-                            ColorTools.GetTextColor(eventColor),
+                        TextColor eventTextColor = TextColorTools.GetBestTextColor(
+                            ColorConverter.GetTextColor(eventColor),
                             bgTextColor,
-                            ColorContrast.High
+                            TextColorContrast.High
                         );
-                        eventTag.ForegroundGdk = ColorTools.GetGdkColor(
+                        eventTag.ForegroundGdk = ColorConverter.GetGdkColor(
                             eventTextColor
                         );
-                        Buffer.InsertWithTagsByName(ref iter, timestamp, "event");
+                        buffer.InsertWithTagsByName(ref iter, timestamp, "event");
                     } else {
-                        Buffer.Insert(ref iter, timestamp);
+                        buffer.Insert(ref iter, timestamp);
                     }
                 }
             }
@@ -241,16 +255,39 @@ namespace Smuxi.Frontend.Gnome
 
                 // TODO: implement all types
                 if (msgPart is UrlMessagePartModel) {
-                    UrlMessagePartModel fmsgui = (UrlMessagePartModel) msgPart;
+                    var urlPart = (UrlMessagePartModel) msgPart;
                     // HACK: the engine should set a color for us!
-                    Gtk.TextTag urlTag = _MessageTextTagTable.Lookup("url");
-                    Gdk.Color urlColor = urlTag.ForegroundGdk;
-                    //Console.WriteLine("urlColor: " + urlColor);
-                    TextColor urlTextColor = ColorTools.GetTextColor(urlColor);
-                    urlTextColor = ColorTools.GetBestTextColor(urlTextColor, bgTextColor);
-                    //Console.WriteLine("GetBestTextColor({0}, {1}): {2}",  urlColor, bgTextColor, urlTextColor);
-                    urlTag.ForegroundGdk = ColorTools.GetGdkColor(urlTextColor);
-                    Buffer.InsertWithTagsByName(ref iter, fmsgui.Url, "url");
+                    var linkStyleTag = _MessageTextTagTable.Lookup("link");
+                    var linkColor = ColorConverter.GetTextColor(
+                        linkStyleTag.ForegroundGdk
+                    );
+                    linkColor = TextColorTools.GetBestTextColor(
+                        linkColor, bgTextColor
+                    );
+                    var linkText = urlPart.Text ?? urlPart.Url;
+
+                    var url = urlPart.Url;
+                    // HACK: assume http if no protocol/scheme was specified
+                    if (urlPart.Protocol == UrlProtocol.None ||
+                        !Regex.IsMatch(url, @"^[a-zA-Z0-9\-]+:")) {
+                        url = String.Format("http://{0}", url);
+                    }
+                    Uri uri;
+                    try {
+                        uri = new Uri(url);
+                    } catch (UriFormatException ex) {
+#if LOG4NET
+                        _Logger.Error("AddMessage(): Invalid URL: " + url, ex);
+#endif
+                        buffer.Insert(ref iter, linkText);
+                        continue;
+                    }
+
+                    var linkTag = new LinkTag(uri);
+                    linkTag.ForegroundGdk = ColorConverter.GetGdkColor(linkColor);
+                    linkTag.TextEvent += OnLinkTagTextEvent;
+                    _MessageTextTagTable.Add(linkTag);
+                    buffer.InsertWithTags(ref iter, linkText, linkStyleTag, linkTag);
                 } else if (msgPart is TextMessagePartModel) {
                     TextMessagePartModel fmsgti = (TextMessagePartModel) msgPart;
                     List<string> tags = new List<string>();
@@ -259,7 +296,9 @@ namespace Smuxi.Frontend.Gnome
                         if (fmsgti.BackgroundColor != TextColor.None) {
                             bg = fmsgti.BackgroundColor;
                         }
-                        TextColor color = ColorTools.GetBestTextColor(fmsgti.ForegroundColor, bg);
+                        TextColor color = TextColorTools.GetBestTextColor(
+                            fmsgti.ForegroundColor, bg
+                        );
                         //Console.WriteLine("GetBestTextColor({0}, {1}): {2}",  fmsgti.ForegroundColor, bgTextColor, color);
                         string tagname = GetTextTagName(color, null);
                         //string tagname = _GetTextTagName(fmsgti.ForegroundColor, null);
@@ -271,19 +310,19 @@ namespace Smuxi.Frontend.Gnome
                         tags.Add(tagname);
                     }
                     if (fmsgti.Underline) {
-#if LOG4NET
+#if LOG4NET && MSG_DEBUG
                         _Logger.Debug("AddMessage(): fmsgti.Underline is true");
 #endif
                         tags.Add("underline");
                     }
                     if (fmsgti.Bold) {
-#if LOG4NET
+#if LOG4NET && MSG_DEBUG
                         _Logger.Debug("AddMessage(): fmsgti.Bold is true");
 #endif
                         tags.Add("bold");
                     }
                     if (fmsgti.Italic) {
-#if LOG4NET
+#if LOG4NET && MSG_DEBUG
                         _Logger.Debug("AddMessage(): fmsgti.Italic is true");
 #endif
                         tags.Add("italic");
@@ -295,14 +334,14 @@ namespace Smuxi.Frontend.Gnome
                     }
 
                     if (tags.Count > 0) {
-                        Buffer.InsertWithTagsByName(ref iter, fmsgti.Text, tags.ToArray());
+                        buffer.InsertWithTagsByName(ref iter, fmsgti.Text, tags.ToArray());
                     } else {
-                        Buffer.Insert(ref iter, fmsgti.Text);
+                        buffer.Insert(ref iter, fmsgti.Text);
                     }
                 }
             }
             if (addLinebreak) {
-                Buffer.Insert(ref iter, "\n");
+                buffer.Insert(ref iter, "\n");
             }
 
             CheckBufferSize();
@@ -364,10 +403,9 @@ namespace Smuxi.Frontend.Gnome
             tt.Underline = Pango.Underline.Single;
             ttt.Add(tt);
             
-            tt = new Gtk.TextTag("url");
+            tt = new Gtk.TextTag("link");
             tt.Underline = Pango.Underline.Single;
             tt.Foreground = "darkblue";
-            tt.TextEvent += OnTextTagUrlTextEvent;
             fd = new Pango.FontDescription();
             tt.FontDesc = fd;
             ttt.Add(tt);
@@ -396,15 +434,15 @@ namespace Smuxi.Frontend.Gnome
             Gtk.TextIter iter = GetIterAtLocation(bufferX, bufferY);
             bool atUrlTag = false;
             foreach (Gtk.TextTag tag in iter.Tags) {
-                if (tag.Name == "url") {
+                if (tag.Name == "link") {
                     atUrlTag = true;
                     break;
                 }
             }
             
             Gdk.Window window = GetWindow(Gtk.TextWindowType.Text); 
-            if (atUrlTag != _AtUrlTag) {
-                _AtUrlTag = atUrlTag;
+            if (atUrlTag != _AtLinkTag) {
+                _AtLinkTag = atUrlTag;
                 
                 if (atUrlTag) {
 #if LOG4NET
@@ -416,54 +454,46 @@ namespace Smuxi.Frontend.Gnome
                     _Logger.Debug("OnMotionNotifyEvent(): not at url tag");
 #endif
                     window.Cursor = _NormalCursor;
+                    _ActiveLink = null;
                 }
             }
         }
         
-        protected virtual void OnTextTagUrlTextEvent(object sender, Gtk.TextEventArgs e)
+        protected virtual void OnLinkTagTextEvent(object sender, Gtk.TextEventArgs e)
         {
             // logging noise
             //Trace.Call(sender, e);
             
-            Gtk.TextIter start = Gtk.TextIter.Zero;
-            Gtk.TextIter end = Gtk.TextIter.Zero;
-
             // if something in the textview is selected, bail out
             if (HasTextViewSelection) {
 #if LOG4NET
-                _Logger.Debug("OnTextTagUrlTextEvent(): active selection present, bailing out...");
+                _Logger.Debug("OnLinkTagTextEvent(): active selection present, bailing out...");
 #endif
                 return;
             }
             
-            // get URL via TextTag from TextIter
-            Gtk.TextTag tag = (Gtk.TextTag) sender;
-            
-            start = e.Iter;
-            start.BackwardToTagToggle(tag);
-            end = e.Iter;
-            end.ForwardToTagToggle(tag);
-            _Url = Buffer.GetText(start, end, false);
+            var tag = (LinkTag) sender;
+            _ActiveLink = tag.Link;
 
             if (e.Event.Type != Gdk.EventType.ButtonRelease) {
                 return;
             }
 
-            if (String.IsNullOrEmpty(_Url)) {
+            if (_ActiveLink == null) {
 #if LOG4NET
-                _Logger.Warn("OnTextTagUrlTextEvent(): url is empty, ignoring...");
+                _Logger.Warn("OnLinkTagTextEvent(): _ActiveLink is null, ignoring...");
 #endif
                 return;
             }
 
-            OpenLink(_Url);
+            OpenLink(_ActiveLink);
         }
 
         protected virtual void OnPopulatePopup(object sender, Gtk.PopulatePopupArgs e)
         {
             Trace.Call(sender, e);
 
-            if (!_AtUrlTag) {
+            if (!_AtLinkTag) {
                 return;
             }
 
@@ -475,8 +505,8 @@ namespace Smuxi.Frontend.Gnome
 
             Gtk.ImageMenuItem open_item = new Gtk.ImageMenuItem(Gtk.Stock.Open, null);
             open_item.Activated += delegate {
-                if (!String.IsNullOrEmpty(_Url)) {
-                    OpenLink(_Url);
+                if (_ActiveLink != null) {
+                    OpenLink(_ActiveLink);
                 }
             };
             popup.Append(open_item);
@@ -485,32 +515,38 @@ namespace Smuxi.Frontend.Gnome
             copy_item.Activated += delegate {
                 Gdk.Atom clipboardAtom = Gdk.Atom.Intern("CLIPBOARD", false);
                 Gtk.Clipboard clipboard = Gtk.Clipboard.Get(clipboardAtom);
-                clipboard.Text = _Url;
+                clipboard.Text = _ActiveLink.ToString();
             };
             popup.Append(copy_item);
 
             popup.ShowAll();
         }
 
-        private void OpenLink(string link)
+        private void OpenLink(Uri link)
         {
             Trace.Call(link);
 
-            if (!Regex.IsMatch(link, @"^[a-zA-Z0-9\-]+:\/\/")) {
-                // URL doesn't start with a protocol
-                link = "http://" + link;
+            if (link == null) {
+                throw new ArgumentNullException("link");
             }
-            
+
             // hopefully MS .NET / Mono finds some way to handle the URL
             ThreadPool.QueueUserWorkItem(delegate {
+                var url = link.ToString();
                 try {
-                    SysDiag.Process.Start(link);
+                    using (var process = SysDiag.Process.Start(url)) {
+                        // Start() might return null in case it re-used a
+                        // process instead of starting one
+                        if (process != null) {
+                            process.WaitForExit();
+                        }
+                    }
                 } catch (Exception ex) {
                     // exceptions in the thread pool would kill the process, see:
                     // http://msdn.microsoft.com/en-us/library/0ka9477y.aspx
                     // http://projects.qnetp.net/issues/show/194
 #if LOG4NET
-                    _Logger.Error("OpenLink(): opening URL: '" + link + "' failed", ex);
+                    _Logger.Error("OpenLink(): opening URL: '" + url + "' failed", ex);
 #endif
                 }
             });
@@ -541,9 +577,6 @@ namespace Smuxi.Frontend.Gnome
                  } else if (bgColor != null) {
                     tt.BackgroundGdk = c;
                  }
-#if LOG4NET
-                 _Logger.Debug("GetTextTagName(): adding: " + tagname + " to _OutputTextTagTable");
-#endif
                  _MessageTextTagTable.Add(tt);
              }
              return tagname;
