@@ -1,13 +1,7 @@
 /*
- * $Id: Frontend.cs 192 2007-04-22 11:48:12Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/Frontend.cs $
- * $Rev: 192 $
- * $Author: meebey $
- * $Date: 2007-04-22 13:48:12 +0200 (Sun, 22 Apr 2007) $
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2007 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2007, 2010-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -32,7 +26,7 @@ using System.Reflection;
 using System.Runtime.InteropServices;
 using Smuxi.Engine;
 using Smuxi.Common;
-using STFL = Stfl;
+using Stfl;
 
 namespace Smuxi.Frontend.Stfl
 {
@@ -128,10 +122,10 @@ namespace Smuxi.Frontend.Stfl
             }
         }
         
-        public static void Init(string[] args)
+        public static void Init(string engine)
         {
             System.Threading.Thread.CurrentThread.Name = "Main";
-            Trace.Call(args);
+            Trace.Call(engine);
            
             Assembly asm = Assembly.GetAssembly(typeof(Frontend));
             AssemblyName asm_name = asm.GetName(false);
@@ -141,7 +135,8 @@ namespace Smuxi.Frontend.Stfl
             _VersionNumber = asm_name.Version.ToString();
             _VersionString = pr.Product + " - " + _UIName + " frontend " + _Version;
 
-            //STFL.error_action("print");
+            // this always calls abort() :(((
+            //StflApi.stfl_error_action("print");
             
 #if LOG4NET
             _Logger.Info(_VersionString + " starting");
@@ -152,16 +147,14 @@ namespace Smuxi.Frontend.Stfl
             // loading and setting defaults
             _FrontendConfig.Load();
             _FrontendConfig.Save();
-           
+
             if (_FrontendConfig.IsCleanConfig) {
                 // first start assistant
             } else {
-                if (((string)FrontendConfig["Engines/Default"]).Length == 0) {
+                if (String.IsNullOrEmpty(engine) || engine == "local") {
                     InitLocalEngine();
                 } else {
-                    // there is a default engine set, means we want a remote engine
-                    //new EngineManagerDialog();
-                    InitLocalEngine();
+                    InitRemoteEngine(engine);
                 }
             }
             
@@ -187,6 +180,42 @@ namespace Smuxi.Frontend.Stfl
             ConnectEngineToGUI();
         }
         
+        public static void InitRemoteEngine(string engine)
+        {
+            var manager = new EngineManager(_FrontendConfig,
+                                            _MainWindow.UI);
+            try {
+                try {
+                    Console.WriteLine(
+                        _("Connecting to remote engine '{0}'..."), engine
+                    );
+                    manager.Connect(engine);
+                    Console.WriteLine(_("Connection established"));
+                } catch (Exception ex) {
+#if LOG4NET
+                    _Logger.Error(ex);
+#endif
+                    Console.WriteLine(
+                        _("Connection failed! Error: {1}"),
+                        engine,
+                        ex.Message
+                    );
+                    Environment.Exit(1);
+                }
+
+                _Session = manager.Session;
+                _UserConfig = manager.UserConfig;
+                _EngineVersion = manager.EngineVersion;
+                ConnectEngineToGUI();
+            } catch (Exception ex) {
+#if LOG4NET
+                _Logger.Error(ex);
+#endif
+                manager.Disconnect();
+                throw;
+            }
+        }
+
         public static void ConnectEngineToGUI()
         {
             _FrontendManager = _Session.GetFrontendManager(_MainWindow.UI);
@@ -233,5 +262,10 @@ namespace Smuxi.Frontend.Stfl
         {
             //Application.Error("Error occurred!", ex.ToString());
         }
+
+        static string _(string msg)
+        {
+            return Mono.Unix.Catalog.GetString(msg);
+        }
     }
 }
