@@ -65,23 +65,45 @@ namespace Smuxi.Frontend
             }
             return null;
         }
-        
-        protected IChatView CreateChatView(ChatModel chat)
+
+        [Obsolete("Use CreateChatView(ChatModel, ChatType, Type) instead.")]
+        protected IChatView CreateChatView(ChatModel chat,
+                                           params object[] parameters)
         {
-            Trace.Call(chat);
-            
+            Trace.Call(chat, parameters);
+
+            // REMOTING CALL 1 + 2
+            return CreateChatView(chat, chat.ChatType,
+                                  chat.ProtocolManager.GetType(), parameters);
+        }
+
+        protected IChatView CreateChatView(ChatModel chat,
+                                           ChatType chatType,
+                                           Type protocolManagerType,
+                                           params object[] parameters)
+        {
+            Trace.Call(chat, chatType, protocolManagerType, parameters);
+
             Type type;
-            type = _GetChatViewType(chat.ChatType, chat.ProtocolManager != null ?
-                                                   chat.ProtocolManager.GetType() : null);
+            type = _GetChatViewType(chatType, protocolManagerType);
             if (type == null) {
-                type = _GetChatViewType(chat.ChatType, null);
+                type = _GetChatViewType(chatType, null);
             }
             
             if (type == null) {
                 throw new ApplicationException("Unsupported ChatModel type: " + chat.GetType());
             }
             
-            return (IChatView) Activator.CreateInstance(type, chat);
+            object[] ctorParams;
+            if (parameters != null && parameters.Length > 0) {
+                ctorParams = new object[parameters.Length + 1];
+                ctorParams[0] = chat;
+                parameters.CopyTo(ctorParams, 1);
+            } else {
+                ctorParams = new object[] {chat};
+            }
+
+            return (IChatView) Activator.CreateInstance(type, ctorParams);
         }
         
         public void LoadAll(string path, string pattern)
