@@ -114,7 +114,25 @@ namespace Smuxi.Frontend
         public void Connect(string engine)
         {
             Trace.Call(engine);
-            
+
+            if (engine == null) {
+                throw new ArgumentNullException("engine");
+            }
+            if (engine.Length == 0) {
+                throw new ArgumentException(_("Engine must not be empty."), "engine");
+            }
+
+            bool engineFound = false;
+            foreach (var entry in (string[]) f_FrontendConfig["Engines/Engines"]) {
+                if (entry == engine) {
+                    engineFound = true;
+                    break;
+                }
+            }
+            if (!engineFound) {
+                throw new ArgumentException(_("Engine does not exist."), "engine");
+            }
+
             f_Engine = engine;
             string username = (string) f_FrontendConfig["Engines/"+engine+"/Username"];
             string password = (string) f_FrontendConfig["Engines/"+engine+"/Password"];
@@ -138,7 +156,15 @@ namespace Smuxi.Frontend
             }
             string sshUsername = (string) f_FrontendConfig["Engines/"+engine+"/SshUsername"];
             string sshPassword = (string) f_FrontendConfig["Engines/"+engine+"/SshPassword"];
-            
+            var sshKeyfile = (string) f_FrontendConfig["Engines/"+engine+"/SshKeyfile"];
+
+            // OPT: always use SSH compression (both openssh and plink support it)
+            // this reduces the .NET remoting traffic by about 75%
+            if (String.IsNullOrEmpty(sshParameters) ||
+                !sshParameters.Contains(" -C")) {
+                sshParameters += " -C";
+            }
+
             int remotingPort = 0;
             if (useSshTunnel) {
                 // find free remoting back-channel port
@@ -165,8 +191,8 @@ namespace Smuxi.Frontend
                 // thus the client will try to reach it using the original
                 // server port :(
                 f_SshTunnelManager = new SshTunnelManager(
-                    sshProgram, sshParameters, sshUsername, sshPassword, null,
-                    sshHostname, sshPort,
+                    sshProgram, sshParameters, sshUsername, sshPassword,
+                    sshKeyfile, sshHostname, sshPort,
                     //"127.0.0.1", localForwardPort, "127.0.0.1", port,
                     "127.0.0.1", port, "127.0.0.1", port,
                     "127.0.0.1", remotingPort, "127.0.0.1", remotingPort
