@@ -89,6 +89,7 @@ namespace Smuxi.Frontend
             f_Parameters = parameters;
             f_Username = username;
             f_Password = password;
+            f_Keyfile = keyfile;
             f_Hostname = hostname;
             f_Port = port;
             
@@ -122,10 +123,8 @@ namespace Smuxi.Frontend
         {
             Trace.Call(disposing);
             
-            if (disposing) {
-                if (f_Process != null) {
-                    f_Process.Dispose();
-                }
+            if (f_Process != null) {
+                f_Process.Dispose();
             }
         }
         
@@ -326,6 +325,19 @@ namespace Smuxi.Frontend
             if (!String.IsNullOrEmpty(f_Password)) {
                 // TODO: pass password,  but how?
             }
+            if (!String.IsNullOrEmpty(f_Keyfile)) {
+                if (!File.Exists(f_Keyfile)) {
+                    throw new ApplicationException(_("SSH keyfile not found."));
+                }
+                try {
+                    using (File.OpenRead(f_Keyfile)) {}
+                } catch (Exception ex) {
+                    throw new ApplicationException(
+                        _("SSH keyfile could not be read."), ex
+                    );
+                }
+                sshArguments += String.Format(" -i \"{0}\"", f_Keyfile);
+            }
             if (f_Port != -1) {
                 sshArguments += String.Format(" -p {0}", f_Port);
             }
@@ -372,9 +384,16 @@ namespace Smuxi.Frontend
             psi.RedirectStandardOutput = true;
             psi.RedirectStandardError = true;
 
-            SysDiag.Process process = SysDiag.Process.Start(psi);
-            string error = process.StandardError.ReadToEnd();
-            string output = process.StandardOutput.ReadToEnd();
+            string error;
+            string output;
+            int exitCode;
+            using (var process = SysDiag.Process.Start(psi)) {
+                error = process.StandardError.ReadToEnd();
+                output = process.StandardOutput.ReadToEnd();
+                process.WaitForExit();
+                exitCode = process.ExitCode;
+            }
+
             string haystack;
             // we expect the version output on stderr
             if (error.Length > 0) {
@@ -408,7 +427,7 @@ namespace Smuxi.Frontend
                   "{2}\n" +
                   "Program Output:\n" +
                   "{3}\n"),
-                f_Process.ExitCode,
+                exitCode,
                 f_Program,
                 error,
                 output
@@ -423,11 +442,18 @@ namespace Smuxi.Frontend
         {
             string sshArguments = String.Empty;
             
-            // HACK: don't ask for SSH key fingerprints
-            // this is nasty but plink.exe can't ask for fingerprint
-            // confirmation and thus the connect would always fail
-            sshArguments += " -auto_store_key_in_cache";
-            
+            var sshVersion = GetPlinkVersionString();
+            // Smuxi by default ships Plink of Quest PuTTY which allows to
+            // accept any fingerprint but does _not_ work with pagent thus we
+            // need to also support the regular plink if the user wants
+            // ssh key authentication instead
+            if (sshVersion.EndsWith("_q1.129")) {
+                // HACK: don't ask for SSH key fingerprints
+                // this is nasty but plink.exe can't ask for fingerprint
+                // confirmation and thus the connect would always fail
+                sshArguments += " -auto_store_key_in_cache";
+            }
+
             // no interactive mode please
             sshArguments += " -batch";
             // don't execute a remote command
@@ -445,7 +471,19 @@ namespace Smuxi.Frontend
             if (!String.IsNullOrEmpty(f_Password)) {
                 sshArguments += String.Format(" -pw {0}", f_Password);
             }
-            
+            if (!String.IsNullOrEmpty(f_Keyfile)) {
+                if (!File.Exists(f_Keyfile)) {
+                    throw new ApplicationException(_("SSH keyfile not found."));
+                }
+                try {
+                    using (File.OpenRead(f_Keyfile)) {}
+                } catch (Exception ex) {
+                    throw new ApplicationException(
+                        _("SSH keyfile could not be read."), ex
+                    );
+                }
+                sshArguments += String.Format(" -i \"{0}\"", f_Keyfile);
+            }
             if (f_Port != -1) {
                 sshArguments += String.Format(" -P {0}", f_Port);
             }
@@ -484,6 +522,53 @@ namespace Smuxi.Frontend
             return psi;
         }
 
+        private string GetPlinkVersionString()
+        {
+            var startInfo = new SysDiag.ProcessStartInfo() {
+                FileName = f_Program,
+                Arguments = "-V",
+                UseShellExecute = false,
+                RedirectStandardOutput = true,
+                RedirectStandardError = true,
+                CreateNoWindow = true
+            };
+            string error;
+            string output;
+            int exitCode;
+            using (var process = SysDiag.Process.Start(startInfo)) {
+                error = process.StandardError.ReadToEnd();
+                output = process.StandardOutput.ReadToEnd();
+                process.WaitForExit();
+                exitCode = process.ExitCode;
+            }
+
+            Match match = Regex.Match(output, @"[0-9]+\.[0-9a-zA-Z_]+");
+            if (match.Success) {
+                var version = match.Value;
+#if LOG4NET
+                f_Logger.Debug("GetPlinkVersionString(): found version: " + version);
+#endif
+                return version;
+            }
+
+            string msg = String.Format(
+                _("Plink version number not found (exit code: {0})\n\n" +
+                  "SSH program: {1}\n\n" +
+                  "Program Error:\n" +
+                  "{2}\n" +
+                  "Program Output:\n" +
+                  "{3}\n"),
+                exitCode,
+                f_Program,
+                error,
+                output
+            );
+#if LOG4NET
+            f_Logger.Error("GetPlinkVersionString(): " + msg);
+#endif
+            throw new ApplicationException(msg);
+        }
+
         private static string _(string msg)
         {
             return LibraryCatalog.GetString(msg, f_LibraryTextDomain);
